import re

def find_all(a_str, sub):
    start = 0
    while True:
        start = a_str.find(sub, start)
        if start == -1: return
        yield start
        start += len(sub) # use start += 1 to find overlapping matches

def parse_vcf(vcf_file_name, vcf_lines, vcf_headers, add_sname=True):
    header = ''
    samples = ''

    f = open(vcf_file_name, 'r')

    for l in f:
        if l[0] == '#':
            if l[1] != '#':
                samples = l.rstrip().split('\t')[9:]
            else:
                # ignore fileDate
                if l[:10] == '##fileDate':
                    continue
                if l not in vcf_headers:
                    vcf_headers.append(l)
        else:
            A = l.split('\t')
            if not 'SECONDARY' in A[7]:

                if add_sname and (samples != ''):
                    A[7] += ';' + 'SNAME=' + ','.join(samples)
                    l = '\t'.join(A)

                
                if 'SVTYPE=BND' in A[7]:
                    m = re.search(r"(\[|\])(.*)(\[|\])",A[4])
                    o_chr,o_pos = m.group(2).split(':')

                    if (o_chr == A[0]) and (('--:' in A[7]) != ('++' in A[7])):
                        neg_s = A[7].find('--:')
                        pos_s = A[7].find('++:')

                        if neg_s > 0:
                            neg_e = neg_s + A[7][neg_s:].find(';') 
                            pre=A[7][:neg_s]
                            mid=A[7][neg_s:neg_e]
                            post=A[7][neg_e:]
                            A[7] = pre + '++:0,' + mid + post
                        else:
                            pos_e = pos_s + A[7][pos_s:].find(';') 
                            pre=A[7][:pos_s]
                            mid=A[7][pos_s:pos_e]
                            post=A[7][pos_e:]
                            A[7] = pre + mid + ',--:0' + post

                        A[7] = 'SVTYPE=INV' + A[7][10:] + ';END=' + o_pos
                        A[4] = '<INV>'
                        l = '\t'.join(A)
                vcf_lines.append(l)

    return samples




def parse_vcf_record(vcf_line):

    A = vcf_line.split('\t')
    if not 'SECONDARY' in A[7]:

        if 'SVTYPE=BND' in A[7]:
            m = re.search(r"(\[|\])(.*)(\[|\])",A[4])
            o_chr, o_pos=m.group(2).split(':')

            if (o_chr == A[0]) and (('--:' in A[7]) != ('++' in A[7])):
                neg_s = A[7].find('--:')
                pos_s = A[7].find('++:')

                if neg_s > 0:
                    neg_e = neg_s + A[7][neg_s:].find(';')
                    pre=A[7][:neg_s]
                    mid=A[7][neg_s:neg_e]
                    post=A[7][neg_e:]
                    A[7] = pre + '++:0,' + mid + post
                else:
                    pos_e = pos_s + A[7][pos_s:].find(';')
                    pre=A[7][:pos_s]
                    mid=A[7][pos_s:pos_e]
                    post=A[7][pos_e:]
                    A[7] = pre + mid + ',--:0' + post

                A[7] = 'SVTYPE=INV' + A[7][10:] + ';END=' + o_pos
                A[4] = '<INV>'
                vcf_line='\t'.join(A)

    return vcf_line


def split_v(l):
    A = l.split('\t', 8)
    m = to_map(A[7])

    chr_l = A[0]
    pos_l = int(A[1])

    chr_r = A[0]
    pos_r = int(A[1])
    if m['SVTYPE'] == 'BND':
        sep = '['
        if not sep in A[4]:
            sep = ']'
        s,e = [x for x in find_all(A[4],sep)]
        chr_r,pos_r = A[4][s+1:e].split(':')
        m['END'] = pos_r
        pos_r = int(pos_r)
    else:
        pos_r = int(m['END'])

    start_l = pos_l + int(m['CIPOS'].split(',')[0])
    end_l = pos_l + int(m['CIPOS'].split(',')[1])

    start_r = pos_r + int(m['CIEND'].split(',')[0])
    end_r = pos_r + int(m['CIEND'].split(',')[1])
        
    strands = m['STRANDS']

    return [m['SVTYPE'],chr_l,chr_r,strands,start_l,end_l,start_r,end_r,m]

def to_map(s):
    m = {}
    for k_v in s.split(';'):
        A = k_v.split('=')
        if len(A) > 1:
            m[A[0]] = A[1]
        else:
            m[A[0]] = None

    return m

def vcf_line_key(l1):
    v1 = split_v(l1)[:8]
    v1[3] = v1[3][:2]
    return v1


def vcf_line_cmp(l1, l2):
    v1 = split_v(l1)
    v2 = split_v(l2)

    v1[3] = v1[3][:2]
    v2[3] = v2[3][:2]

    for i in range(len(v1)-1):
        if v1[i] != v2[i]:
            return cmp(v1[i],v2[i])
    return 0

def header_line_cmp(l1, l2):
    order = ['##source', \
             '##INFO', \
             '##ALT', \
             '##FORMAT',\
             '##SAMPLE']

    # make sure ##fileformat is first
    if l1[:12] == '##fileformat':
        return -1

    if l2[:12] == '##fileformat':
        return 1 

    # make sure #CHROM ... is last
    if l1[1] != '#':
        return 1
    elif l2[1] != '#':
        return -1

    if l1.find('=') == -1:
        return -1 
    if l2.find('=') == -1:
        return 1

    h1 = l1[:l1.find('=')]
    h2 = l2[:l2.find('=')]
    if h1 not in order:
        return -1 
    if h2 not in order:
        return 1
    return cmp(order.index(h1),order.index(h2))

class breakpoint:
    chr_l = ''
    start_l = 0
    end_l = 0
    p_l = []

    chr_r = ''
    start_r = 0
    end_r = 0
    p_r = []

    sv_type = ''

    strands = ''
    
    l = ''

    def __init__(self, 
                 l,
                 percent_slop=0,
                 fixed_slop=0):
        self.l = l

        [self.sv_type,\
        self.chr_l, \
        self.chr_r,\
        self.strands,
        self.start_l,\
        self.end_l,\
        self.start_r, \
        self.end_r, 
        m] = split_v(l)

        self.p_l = [float(x) for x in m['PRPOS'].split(',')]
        self.p_r = [float(x) for x in m['PREND'].split(',')]

        slop_prob = 1e-100
        if ((percent_slop > 0) or (fixed_slop > 0)):

            l_slop = int(max(percent_slop*(self.end_l - self.start_l + 1),fixed_slop))
            r_slop = int(max(percent_slop*(self.end_r - self.start_r + 1),fixed_slop))

            # pad each interval with slop_prob on each side.
            self.start_l = self.start_l-l_slop
            self.end_l = self.end_l+l_slop
            new_p_l = [slop_prob] * l_slop + self.p_l + [slop_prob] * l_slop

            self.start_r = self.start_r-r_slop
            self.end_r = self.end_r+r_slop
            new_p_r = [slop_prob] * r_slop + self.p_r + [slop_prob] * r_slop

            # chew off overhang if self.start_l or self.start_r less than 0
            if self.start_l < 0:
                new_p_l = new_p_l[-self.start_l:]
                self.start_l = 0
            if self.start_r < 0:
                new_p_r = new_p_r[-self.start_r:]
                self.start_r = 0

            # normalize so each probability curve sums to 1.
            sum_p_l = sum(new_p_l)
            self.p_l = [float(x)/sum_p_l for x in new_p_l]
            sum_p_r = sum(new_p_r)
            self.p_r = [float(x)/sum_p_r for x in new_p_r]

            # old_l = float(self.end_l - self.start_l + 1)
            
            # self.start_l = max(0,self.start_l-l_slop)
            # self.end_l = self.end_l+l_slop

            # new_l = float(self.end_l - self.start_l + 1)

            # new_p_l = []
            # for i in range(self.end_l-self.start_l+1):
            #     p = i/new_l
            #     old_i = int(p*old_l)
            #     new_p_l.append(self.p_l[old_i])
            # sum_p_l = sum(new_p_l)
            # self.p_l = [float(x)/sum_p_l for x in new_p_l]

            # old_r = float(self.end_r - self.start_r + 1)

            # self.start_r = max(0,self.start_r-r_slop)
            # self.end_r = self.end_r+r_slop

            # new_r = float(self.end_r - self.start_r + 1)

            # new_p_r = []
            # for i in range(self.end_r-self.start_r+1):
            #     p = float(i)/new_r
            #     old_i = int(p*old_r)
            #     new_p_r.append(self.p_r[old_i])
            # sum_p_r = max(1,sum(new_p_r))
            # self.p_r = [float(x)/sum_p_r for x in new_p_r]

    def __str__(self):
        return '\t'.join([str(x) for x in [self.chr_l, \
                                           self.start_l,\
                                           self.end_l,\
                                           self.chr_r,\
                                           self.start_r, \
                                           self.end_r, 
                                           self.sv_type,\
                                           self.strands,\
                                           self.p_l,
                                           self.p_r]])
    def ovl(self, b):
        if (self.chr_l != b.chr_l) or \
            (self.chr_r != b.chr_r) or \
            (self.sv_type != b.sv_type):
            return 0
        #get left common interval
        c_start_l, c_end_l = [max(self.start_l, b.start_l), \
                              min(self.end_l, b.end_l)]
        #get right common interval
        c_start_r, c_end_r = [max(self.start_r, b.start_r), \
                              min(self.end_r, b.end_r)]

        c_l_len = c_end_l - c_start_l + 1
        c_r_len = c_end_r - c_start_r + 1

        if (c_l_len < 1) or (c_r_len < 1):
            return 0

        self_start_off_l = c_start_l - self.start_l
        b_start_off_l = c_start_l - b.start_l

        self_start_off_r = c_start_r - self.start_r
        b_start_off_r = c_start_r - b.start_r

        ovl_l = 0
        for i in range(c_l_len):
            ovl_l += min(self.p_l[i+self_start_off_l], b.p_l[i+b_start_off_l])

        ovl_r = 0
        for i in range(c_r_len):
            ovl_r += min(self.p_r[i+self_start_off_r], b.p_r[i+b_start_off_r])

        return ovl_l * ovl_r

def trim(A):
    clip_start = 0
    for i in range(len(A)):
        if A[i] == 0:
            clip_start += 1
        else:
            break
    clip_end = 0
    for i in range(len(A)-1,-1,-1):
        if A[i] == 0:
            clip_end += 1
        else:
            break               
    return [clip_start, clip_end]


# I has 3 components [[start],[end],[p array]]
def align_intervals(I):
    start = -1
    end = -1
    new_I = []

    START = 0
    END = 1
    P = 2

    # find ends
    for i in I:
        if start == -1:
            start = i[START]
            end = i[END]
        else:
            if i[START] < start:
                start = i[START]

            if i[END] > end:
                end = i[END]

    for i in I:
        new_i = i[P]

        if i[START] > start:
            n = i[START] - start
            new_i = [0]*n + new_i

        if i[END] < end:
            n = end - i[END] 
            new_i = new_i + [0]*n
        
        new_I.append(new_i)
        
    return [start, end, new_I]


class node:
    b = None
    color = -1
    edges = None
    def __init__(self, b):
        self.b = b
        self.color = -1
        self.edges = []

def connect(G, B, t):
    #first we need to add all of the elements in B to the graph

    # each node in the graph has 3 elements
    # 0: breakpoint
    # 1: color
    # 2: list of edges
    #    each edge has 2 elements
    #    0: correspondined node id in G
    #    1: weight (ovl score)

    b_ids = []

    for b in B:
        next_id = len(G)
        b_ids.append(next_id)
        #G[next_id] = [b, -1, []]
        G[next_id] = node(b)

    for i in range(len(B)):
        for j in range(len(B)):
            if i != j:
                ovl = B[i].ovl(B[j])
                if ovl > t:
                    #G[b_ids[i]][2].append([b_ids[j], ovl])
                    G[b_ids[i]].edges.append([b_ids[j], ovl])

def bron_kerbosch(G, R, P, X):
    if (len(P) == 0) and (len(X) == 0):
        yield R
    for v in P:
        V = set([v])
        N = set([g[0] for g in G[v].edges])
    
        for r in bron_kerbosch(G, \
                               R.union(V), \
                               P.intersection(N), 
                               X.intersection(N)):
            yield r

        P = P - V
        X = X.union(V)



from svtools_star.svtools.vcf.genotype import Genotype
import sys

class Variant(object):
    '''
    Class for storing information stored in a VCF line
    '''
    def __init__(self, var_list, vcf):
        '''
        Initialize values.

        If fixed_genotypes is True then a string corresponding to the
        genotype portion of the line is cached for printing later.
        '''
        self.chrom = var_list[0]
        self.pos = int(var_list[1])
        self.var_id = var_list[2]
        self.ref = var_list[3]
        self.alt = var_list[4]
        self.qual = var_list[5]
        self.filter = var_list[6]
        self.sample_list = vcf.sample_list
        self.info_list = vcf.info_list
        self.info = dict()
        self.format_list = vcf.format_list
        self.format_set = {i.id for i in vcf.format_list}
        self.gts = None

        # fill in empty sample genotypes
        if len(var_list) < 8:
            sys.stderr.write('\nError: VCF file must have at least 8 columns\n')
            exit(1)

        # make a genotype for each sample at variant
        self.format_string = var_list[8]
        self.format_dict = { key: index for index, key in enumerate(self.format_string.split(':')) }
        self.format_dict.setdefault('GT', len(self.format_dict)) #add GT if it doesn't exist
        self.gts_string = '\t'.join(var_list[9:])

        self.info = dict()
        i_split = [a.split('=') for a in var_list[7].split(';')] # temp list of split info column
        for i in i_split:
            if len(i) == 1:
                i.append(True)
            self.info[i[0]] = i[1]

    def _parse_genotypes(self, genotype_array):
        '''
        Parse the genotype strings
        '''
        gts = dict()
        for index, sample_string in enumerate(genotype_array):
            sample_name = self.sample_list[index]
            sample_field = sample_string.split(':')
            g = Genotype(self, sample_field)
            gts[sample_name] = g
        return gts

    def set_info(self, field, value):
        '''
        Set value of the specified field in the INFO section.
        The INFO field must exist already.
        '''
        if field in [i.id for i in self.info_list]:
            self.info[field] = value
        else:
            sys.stderr.write('\nError: invalid INFO field, \"' + field + '\"\n')
            exit(1)

    def get_info(self, field):
        '''
        Get a value for the given INFO field
        '''
        return self.info[field]

    def get_info_string(self):
        '''
        Construct the INFO string for printing. Order is matched to the header.
        '''
        i_list = list()
        for info_field in self.info_list:
            if info_field.id in self.info.keys():
                if info_field.type == 'Flag':
                    if self.info[info_field.id]:
                        i_list.append(info_field.id)
                else:
                    i_list.append('%s=%s' % (info_field.id, self.info[info_field.id]))
        return ';'.join(i_list)

    def get_format_string(self):
        '''
        Construct the FORMAT field containing the names of the fields in the Genotype columns
        '''
        f_list = list()
        for f in self.format_list:
            if f.id in self.format_dict:
                f_list.append(f.id)
        return ':'.join(f_list)

    def get_gt_string(self, use_cached_gt_string=False):
        '''
        Construct the genotype string.
        '''
        if self.gts:
            if use_cached_gt_string:
                return self.gts_string
            else:
                return '\t'.join(self.genotype(s).get_gt_string() for s in self.sample_list)
        else:
            return self.gts_string

    def _uncache_gts(self):
        '''
        Parse genotypes if they are requested
        '''
        if self.gts is None:
            self.gts = self._parse_genotypes(self.gts_string.split('\t'))
            self.format_string = None

    def genotypes(self):
        '''
        Return a list of all genotype data in the Variant line
        '''
        self._uncache_gts()
        return self.gts.values()

    def genotype(self, sample_name):
        '''
        Return the Genotype object for the requested sample
        '''
        self._uncache_gts()
        try:
            return self.gts[sample_name]
        except KeyError:
            sys.stderr.write('\nError: invalid sample name, \"' + sample_name + '\"\n')
            sys.exit(1)

    def get_var_string(self, use_cached_gt_string=False):
        '''
        Return the String representation for this line
        '''
        fields = [
                self.chrom,
                self.pos,
                self.var_id,
                self.ref,
                self.alt,
                self.qual,
                self.filter,
                self.get_info_string()
                ]

        if self.format_dict:
            gts_string = self.get_gt_string(use_cached_gt_string)
            if gts_string is None:
                sys.stderr.write("Unable to construct or retrieve genotype string\n")
                sys.exit(1)
            else:
                fields += [
                        self.get_format_string(),
                        gts_string
                    ]
        return '\t'.join(map(str, fields))



import argparse, sys
from svtools_star.svtools.bedpe import Bedpe
from svtools_star.svtools.vcf.file import Vcf
from svtools_star.svtools.bedpetovcfconverter import BedpeToVcfConverter
import svtools_star.svtools.utils as su


# primary function
def bedpeToVcf(bedpe_file, vcf_out):
    myvcf = Vcf()
    converter = BedpeToVcfConverter(myvcf)
    in_header = True
    # parse the bedpe data
    header = list()
    for line in bedpe_file:
        if in_header:
            if line[0:2] == '##':
                header.append(line)
                continue
            elif line[0] == '#' and line[1] != '#':
                sample_list_str = line.rstrip().split('\t', 20)[-1]
                header.append('\t'.join([
                                    '#CHROM',
                                    'POS',
                                    'ID',
                                    'REF',
                                    'ALT',
                                    'QUAL',
                                    'FILTER',
                                    'INFO',
                                    sample_list_str
                                    ] ))
                continue
            else:
                in_header = False
                myvcf.add_header(header)
                myvcf.file_format='VCFv4.2'
                vcf_out.write(myvcf.get_header() + '\n')
        #
        bedpe = Bedpe(line.rstrip().split('\t'))
        variants = converter.convert(bedpe)
        for v in variants:
            vcf_out.write(v.get_var_string() + '\n')

    # close the VCF output file and header if no variants found
    if in_header == True:
        myvcf.add_header(header)
        myvcf.file_format='VCFv4.2'
        vcf_out.write(myvcf.get_header() + '\n')
    vcf_out.close()

    return

def description():
    return 'convert a BEDPE file to VCF'

def epilog():
    return 'The input BEDPE file can be gzipped if it is specified explicitly.'

def add_arguments_to_parser(parser):
    parser.add_argument('-b', '--bedpe', metavar='<BEDPE>', default=None, help='BEDPE input (default: stdin)')
    parser.add_argument('-o', '--output', metavar='<VCF>', type=argparse.FileType('w'), default=sys.stdout, help='Output VCF to write (default: stdout)')
    parser.set_defaults(entry_point=run_from_args)

def command_parser():
    parser = argparse.ArgumentParser(description=description())
    add_arguments_to_parser(parser)
    return parser

def run_from_args(args):
    with su.InputStream(args.bedpe) as stream:
        bedpeToVcf(stream, args.output)

if __name__ == '__main__':
    parser = command_parser()
    args = parser.parse_args()
    sys.exit(args.entry_point(args))



#!/usr/bin/env python
# encoding: utf-8

from __future__ import (absolute_import, division, print_function)
from six import reraise as raise_
import sys
import re
import logging
import pandas as pd
import numpy as np
from collections import OrderedDict
import pysam
import starseqr_utils as su

try:
    maketrans = ''.maketrans
except AttributeError:
    # fallback for Python 2
    from string import maketrans


logger = logging.getLogger('STAR-SEQR')


def import_starjxns(jxnFile, keep_dups=False):
    ''' read star jxns file and optionally remove dups'''
    logger.info('Importing junctions')
    try:
        df = pd.read_csv(jxnFile, sep="\t", header=None, usecols=range(0, 14), low_memory=False, engine='c')
        df.columns = ['chrom1', 'pos1', 'str1', 'chrom2', 'pos2', 'str2',
                      'jxntype', 'jxnleft', 'jxnright', 'readid',
                      'base1', 'cigar1', 'base2', 'cigar2']
        df['readid'] = df['readid'].astype(str)
        df['pos1'] = df['pos1'].astype(float).astype(int)  # this bypasses some strange numbers
        df['pos2'] = df['pos2'].astype(float).astype(int)
        df['identity'] = df['base1'].astype(str) + ':' + df['cigar1'].astype(str) + ':' + df['base2'].astype(str) + ':' + df['cigar2'].astype(str)
        df.drop(['base1', 'cigar1', 'base2', 'cigar2'], axis=1, inplace=True)
        if not keep_dups:
            logger.info("Removing duplicate reads")
            return df.drop_duplicates(subset=['identity'], keep='first')
        else:
            logger.info("Using all reads")
            return df
    except Exception as e:
        logger.error("There was a problem reading your STAR *Chimeric.out.junction file")
        logger.error("Exception: " + str(e))
        traceback = sys.exc_info()[2]
        raise_(ValueError, e, traceback)


def choose_order(chrL1, posL1, chrL2, posL2):
    ''' Choose one reprentation of jxn to merge on.
    This is just for DNA breakpoints where bidirectional breakpoint detection occurs'''
    mychrL1 = str(chrL1).replace("chr", "")
    mychrL2 = str(chrL2).replace("chr", "")
    chrList = [mychrL1, mychrL2]
    chrList.sort()
    if ((mychrL1) == (mychrL2)):
        if (int(posL1) < int(posL2)):
            return 1
        else:
            return 2
    elif (int(chrList.index(mychrL1)) < int(chrList.index(mychrL2))):
        return 1
    elif (int(chrList.index(mychrL1)) > int(chrList.index(mychrL2))):
        return 2


def normalize_jxns(chrom1, chrom2, pos1, pos2, strand1, strand2, repleft, repright, order):
    '''Choose one representation for DNA breakpoints'''
    flipstr = maketrans("-+", "+-")
    if order == 2:
        if strand1 == "-":
            new_pos1 = str(chrom1) + ":" + str(pos1 - int(repright)) + ":" + strand1.translate(flipstr)
        else:
            new_pos1 = str(chrom1) + ":" + str(pos1 + int(repright)) + ":" + strand1.translate(flipstr)
        if strand2 == "-":
            new_pos2 = str(chrom2) + ":" + str(pos2 - int(repright)) + ":" + strand2.translate(flipstr)
        else:
            new_pos2 = str(chrom2) + ":" + str(pos2 + int(repright)) + ":" + strand2.translate(flipstr)
        newid = new_pos2 + ":" + new_pos1 + ":" + str(repleft) + ":" + str(repright)
    elif order == 1:
        new_pos1 = str(chrom1) + ":" + str(pos1) + ":" + strand1
        new_pos2 = str(chrom2) + ":" + str(pos2) + ":" + strand2
        newid = new_pos1 + ":" + new_pos2 + ":" + str(repleft) + ":" + str(repright)
    return newid


def count_jxns(df, nucleic_type="RNA"):
    ''' aggregate jxn reads'''
    grouped_df = df.groupby(['name', 'order'], as_index=True)
    new_df = grouped_df['readid'].agg(OrderedDict([('reads', lambda col: ','.join(col)), ('counts', 'count')]))
    new_df = new_df.reset_index().pivot(index='name', columns='order').reset_index()
    if nucleic_type == "DNA":
        new_df.columns = ['name', 'jxnreadsleft', 'jxnreadsright', 'jxnleft', 'jxnright']
    elif nucleic_type == "RNA":
        new_df.columns = ['name', 'jxn_reads', 'jxn_counts']
    return new_df


def get_distance(jxn):
    ''' reference distance '''
    chrom1, pos1, str1, chrom2, pos2, str2, repleft, repright = re.split(':', jxn)
    if chrom1 == chrom2:
        dist_between = abs(int(pos1) - int(pos2))
    else:
        dist_between = np.nan
    return dist_between


def get_pairs_func(jxn, dd):
    '''
    Get paired end read data that supports each jxn from the junction file.
    These have a -1 for jxntype.
    Need to grab both sides of jxn. Each is unique in the .junction file.
    Querying the file can be tedious with the flipping of the junctions.
    Using a dict of pandas dfs to reduce search space and increase performance. 20X increase.
    '''
    chrom1, pos1, str1, chrom2, pos2, str2, repleft, repright = re.split(':', jxn)
    pos1 = int(pos1)
    pos2 = int(pos2)
    maxhom = max(int(repright), int(repleft))
    if str1 == "+":
        pos1left = pos1 - 100000
        pos1right = pos1 + maxhom + 1
    elif str1 == "-":
        pos1left = pos1 - maxhom - 1
        pos1right = pos1 + 100000
    if str2 == "+":
        pos2left = pos2 - maxhom - 1
        pos2right = pos2 + 100000
    elif str2 == "-":
        pos2left = pos2 - 100000
        pos2right = pos2 + maxhom + 1
    forward = dd[chrom1][(dd[chrom1]['chrom2'] == chrom2) &
                         (dd[chrom1]['pos1'] >= pos1left) & (dd[chrom1]['pos1'] <= pos1right) &
                         (dd[chrom1]['pos2'] >= pos2left) & (dd[chrom1]['pos2'] <= pos2right)]
    reverse = dd[chrom2][(dd[chrom2]['chrom2'] == chrom1) &
                         (dd[chrom2]['pos1'] >= pos2left) & (dd[chrom2]['pos1'] <= pos2right) &
                         (dd[chrom2]['pos2'] >= pos1left) & (dd[chrom2]['pos2'] <= pos1right)]
    npairs = len(forward['readid'].index) + len(reverse['readid'].index)
    reads = ','.join(forward['readid'].tolist()) + ',' + ','.join(reverse['readid'].tolist())
    return (npairs, reads)


def flip_jxn(jxn, gs1):
    '''Flip jxn orientation for RNA Fusion that is inverse according to strand info'''
    chrom1, pos1, str1, chrom2, pos2, str2, repleft, repright = re.split(':', jxn)
    chrom1 = str(chrom1)
    chrom2 = str(chrom2)
    pos1 = int(pos1)
    pos2 = int(pos2)
    if str1 != gs1[0]:
        flip = 1
        flipstr = maketrans("-+", "+-")
        if str1 == "-":
            new_pos1 = chrom1 + ":" + str(pos1) + ":" + str1.translate(flipstr)
        else:
            new_pos1 = chrom1 + ":" + str(pos1) + ":" + str1.translate(flipstr)
        if str2 == "-":
            new_pos2 = chrom2 + ":" + str(pos2) + ":" + str2.translate(flipstr)
        else:
            new_pos2 = chrom2 + ":" + str(pos2) + ":" + str2.translate(flipstr)
        newid = new_pos2 + ":" + new_pos1 + ":" + str(repright) + ":" + str(repleft)
    else:
        newid = jxn
        flip = 0
    return (newid, flip)


def exons2seq(fasta_path, lol_exons, jxn, side, fusion_exons='', decorate=''):
    '''
    Given a fasta and a list of exon coordinates, extract sequence.
    exon boundaries can be annotated with a delimiter of somekind
    '''
    clean_jxn = su.common.safe_jxn(jxn)
    jxn_dir = 'support' + '/' + clean_jxn + '/'

    fa = pysam.Fastafile(fasta_path)

    ofile = open(jxn_dir + 'transcripts_' + str(side) + ".fa", "w")
    all_seq = []
    if sum(1 for x in lol_exons if isinstance(x, list)) == 0:
        ofile.close()
        return  # avoid errors if no exons..
    if fusion_exons:
        strand = ''
        fus_strand = ''
        for trx_exons in lol_exons:
            seq = []
            for ex_order, chrom, start, end, strand, trx in trx_exons:  # exon list still ordered in forward
                if fa.references:
                    seq.append(fa.fetch(chrom, start, end))
            if seq:
                seq_str = decorate.join(seq)
                if strand == '-':
                    seq_str = su.common.rc(seq_str)
                for fus_exons in fusion_exons:
                    fus_seq = []
                    for fus_ex_order, fus_chrom, fus_start, fus_end, fus_strand, fus_trx in fus_exons:  # exon list still ordered in forward
                        if fa.references:
                            fus_seq.append(fa.fetch(fus_chrom, fus_start, fus_end))
                    if fus_seq:
                        fus_seq_str = decorate.join(fus_seq)
                        if fus_strand == '-':
                            fus_seq_str = su.common.rc(fus_seq_str)
                        new_trx = str(trx) + "--" + (fus_trx) + "|" + str(len(seq_str))
                        all_seq.append((new_trx, seq_str + fus_seq_str))
                        ofile.write(">" + new_trx + "\n" + seq_str.lower() + fus_seq_str.upper() + "\n")
    else:
        for ntrx_exons in lol_exons:
            nseq = []
            nstrand = ''
            ntrx = ''
            nseq_str = ''
            for nex_order, nchrom, nstart, nend, nstrand, ntrx in ntrx_exons:  # exon list still ordered in forward
                if fa.references:
                    nseq.append(fa.fetch(nchrom, nstart, nend))
            if nseq:
                nseq_str = decorate.join(nseq)
                if nstrand == '-':
                    nseq_str = su.common.rc(nseq_str)
                ofile.write(">" + ntrx + "\n" + nseq_str + "\n")
            all_seq.append((ntrx, nseq_str))  # need empty if no seq found
    ofile.close()
    return pd.Series([[all_seq]])


def get_fusion_class(jxn, txintersection):
    chrom1, pos1, str1, chrom2, pos2, str2, repleft, repright = re.split(':', jxn)
    chrom1 = str(chrom1)
    chrom2 = str(chrom2)
    str1 = str(str1)
    str2 = str(str2)

    if len(str(txintersection)) > 3:  # [] counts as two
        return "GENE_INTERNAL"

    if chrom1 != chrom2:
        return "TRANSLOCATION"

    if chrom1 == chrom2:
        if abs(int(pos1) - int(pos2)) < 1000000:
            return "READ_THROUGH"
        elif str1 == str2:
            return "INTERCHROM_INVERTED"
        elif str1 != str2:
            return "INTERCHROM_INTERSTRAND"


def get_minfrag_length(jxn, df):
    chrom1, pos1, str1, chrom2, pos2, str2, repleft, repright = re.split(':', jxn)
    if str1 == '+':
        left_over = [(int(x) if x else 0) for x in str(df['hangleft_rev_first_seqlen']).split(",")]
        left_jxn = [(int(x) if x else 0) for x in str(df['jxnleft_for_second_seqlen']).split(",")]
    elif str1 == '-':
        left_over = [(int(x) if x else 0) for x in str(df['hangleft_for_first_seqlen']).split(",")]
        left_jxn = [(int(x) if x else 0) for x in str(df['jxnleft_rev_second_seqlen']).split(",")]
    if str2 == '+':
        right_over = [(int(x) if x else 0) for x in str(df['hangright_for_second_seqlen']).split(",")]
        right_jxn = [(int(x) if x else 0) for x in str(df['jxnright_rev_first_seqlen']).split(",")]
    elif str2 == '-':
        right_over = [(int(x) if x else 0) for x in str(df['hangright_rev_second_seqlen']).split(",")]
        right_jxn = [(int(x) if x else 0) for x in str(df['jxnright_for_first_seqlen']).split(",")]
    all_over = left_over + right_over # order matters
    all_jxn = right_jxn + left_jxn # order matters
    all_min = [min(i) for i in list(zip(all_over, all_jxn))] # get the minimum of anchor vs overhang
    all_min20 = np.sum(i > 20 for i in all_min) # how many of the min fragments are > 20
    all_min35 = np.sum(i > 35 for i in all_min) # how many of the min fragments are > 35
    return (all_min20, all_min35)


def get_svtype_func(jxn):
    '''Used for DNA'''
    chrom1, pos1, str1, chrom2, pos2, str2, repleft, repright = re.split(':', jxn)
    chrom1 = str(chrom1)
    chrom2 = str(chrom2)
    str1 = str(str1)
    str2 = str(str2)
    if chrom1 != chrom2:
        svtype = "TRANSLOCATION"
    else:
        # STAR notation is same as other tools after strand2 is flipped.
        if str1 == "+" and str2 == "-":
            svtype = "INVERSION"
        elif str1 == "-" and str2 == "+":
            svtype = "INVERSION"
        elif str1 == "+" and str2 == "+":
            svtype = "DELETION"
        elif str1 == "-" and str2 == "-":
            svtype = "DUPLICATION"
    return svtype


def get_sv_locations(jxn):
    chrom1, pos1, str1, chrom2, pos2, str2, repleft, repright = re.split(':', jxn)
    pos1 = int(pos1) - int(repright)
    pos2 = int(pos2)
    if str(str1) == "+" and str(str2) == "+":
        pos1 -= 1
        pos2 += 1
    elif str(str1) == "-" and str(str2) == "-":
        pos1 += 1
        pos2 -= 1
    elif str(str1) == "+" and str(str2) == "-":
        pos1 -= 1
        pos2 -= 1
    elif str(str1) == "-" and str(str2) == "+":
        pos1 += 1
        pos2 += 1
    brk1 = str(chrom1) + ":" + str(pos1) + ":" + str(str1)
    brk2 = str(chrom2) + ":" + str(pos2) + ":" + str(str2)
    return (brk1, brk2)


def get_fusion_locations(jxn):
    ''' one base coordinates '''
    chrom1, pos1, str1, chrom2, pos2, str2, repleft, repright = re.split(':', jxn)
    brk1 = str(chrom1) + ":" + str(pos1) + ":" + str(str1)
    brk2 = str(chrom2) + ":" + str(pos2) + ":" + str(str2)
    return (brk1, brk2)


def write_bedpe(file_in, file_out):
    with open(file_out, 'w') as file_out_fh:
        with open(file_in, 'r') as starOutput:
            for line in starOutput:
                if not line.startswith(('#','NAME')):
                    vals = line.strip().split()
                    fusion_name = vals[0]
                    valsL = vals[6].split(':')
                    valsR = vals[7].split(':')
                    chrL = valsL[0]
                    posL = int(valsL[1])
                    strandL = valsL[2]
                    chrR = valsR[0]
                    posR = int(valsR[1])
                    strandR = valsR[2]
                    total_uniqreads = int(vals[1]) + int(vals[2]) + int(vals[3])
                    quant = "."
                    bedpe_line = [chrL, posL, posL + 1,
                                  chrR, posR, posR + 1,
                                  fusion_name , total_uniqreads,
                                  strandL, strandR, quant]

                    bedpe = '\t'.join(list(map(str,bedpe_line)))
                    print(bedpe, file=file_out_fh)



import svtools.l_bp as l_bp

import sys
import numpy as np
import argparse

def get_p(ls):
    return np.exp(ls)

def get_ls(p):
    if p == 0:
        return float("-inf")
    else:
        return np.log(p)

def ls_multiply(x, y):
    if (x == float("-inf")) or (y == float("-inf")):
        return float("-inf")
    else:
        return x + y

def ls_divide(x, y):
    return x - y

def ls_add(x, y):
    if x == float("-inf"):
        return y
    elif y == float("-inf"):
        return x
    elif (x < y):
        return y + np.log(1 + np.exp(x - y))
    else:
        return x + np.log(1 + np.exp(y - x))


def print_var_line(l):
    A = l.rstrip().split('\t')

    if A[4] == '<INV>' and ('--:0' in A[7] or '++:0' in A[7]):
        [sv_type,chr_l,chr_r,strands,start_l,end_l,start_r,end_r,m] = \
                l_bp.split_v(l)

        STRAND_DICT = dict(x.split(':') for x in m['STRANDS'].split(','))
        for o in STRAND_DICT.keys():
            if STRAND_DICT[o] == '0':
                del(STRAND_DICT[o])
        STRANDS = ','.join(['%s:%s' % (o,STRAND_DICT[o]) for o in STRAND_DICT])

        if STRANDS[:2] == '++':
            ALT = 'N]' + chr_l + ':' + m['END'] + ']'
        elif STRANDS[:2] == '--':
            ALT = '[' + chr_l + ':' + m['END'] + '[N'

        SVTYPE = 'BND'
        CIPOS = m['CIEND']
        CIEND = m['CIPOS']
        CIPOS95 = m['CIEND95']
        CIEND95 = m['CIPOS95']
        IMPRECISE = 'IMPRECISE'
        SU = m['SU']
        PE = m['PE']
        SR = m['SR']
        PRPOS = m['PREND']
        PREND = m['PRPOS']
        SNAME = m['SNAME']
        ALG = m['ALG']
        EVENT = A[2]

        A[4] = ALT
        A[7] = ';'.join(['SVTYPE='   + str(SVTYPE),
                         'STRANDS='  + str(STRANDS),
                         'CIPOS='    + str(CIPOS),
                         'CIEND='    + str(CIEND),
                         'CIPOS95='  + str(CIPOS95),
                         'CIEND95='  + str(CIEND95),
                                       str(IMPRECISE),
                         'SU='       + str(SU),
                         'PE='       + str(PE),
                         'SR='       + str(SR),
                         'PRPOS='    + str(PRPOS),
                         'PREND='    + str(PREND),
                         'ALG='      + str(ALG),
                         'SNAME='    + str(SNAME),
                         'EVENT='    + str(EVENT)])

        # reconstruct the line
        l = '\t'.join(A)

    if A[4] not in ['<DEL>', '<DUP>', '<INV>']:
        [sv_type,chr_l,chr_r,strands,start_l,end_l,start_r,end_r,m] = \
                l_bp.split_v(l)

        CHROM = chr_r
        POS = m['END']
        ID = A[2] + '_2'
        REF = 'N'
        ALT = ''

        if A[4][0] == '[':
            ALT = '[' + chr_l + ':' + A[1] + '[N'
        elif A[4][0] == ']':
            ALT = 'N[' + chr_l + ':' + A[1] + '['
        elif A[4][-1] == '[':
            ALT = ']' + chr_l + ':' + A[1] + ']N'
        elif A[4][-1] == ']':
            ALT = 'N]' + chr_l + ':' + A[1] + ']'

        QUAL = A[5]
        FILTER = '.'
        SVTYPE = 'BND'
        STRANDS = m['STRANDS']
        CIPOS = m['CIEND']
        CIEND = m['CIPOS']
        CIPOS95 = m['CIEND95']
        CIEND95 = m['CIPOS95']
        IMPRECISE = 'IMPRECISE'
        SU = m['SU']
        PE = m['PE']
        SR = m['SR']
        PRPOS = m['PREND']
        PREND = m['PRPOS']
        SNAME = m['SNAME']
        EVENT = A[2]
        ALG = m['ALG']
        SECONDARY = 'SECONDARY'
        MATEID=A[2] + '_1'

        INFO = ';'.join(['SVTYPE='   + str(SVTYPE),
                         'STRANDS='  + str(STRANDS),
                         'CIPOS='    + str(CIPOS),
                         'CIEND='    + str(CIEND),
                         'CIPOS95='  + str(CIPOS95),
                         'CIEND95='  + str(CIEND95),
                                       str(IMPRECISE),
                                       str(SECONDARY),
                         'SU='       + str(SU),
                         'PE='       + str(PE),
                         'SR='       + str(SR),
                         'PRPOS='    + str(PRPOS),
                         'PREND='    + str(PREND),
                         'ALG='      + str(ALG),
                         'SNAME='    + str(SNAME),
                         'EVENT='    + str(EVENT),
                         'MATEID='   + str(MATEID)])

        O = [CHROM,POS,ID,REF,ALT,QUAL,FILTER,INFO]

        A[7] += ';MATEID=' + A[2] + '_2'
        A[2] += '_1'
        print('\t'.join(A[:8]))
        print('\t'.join([str(o) for o in O]))

    else:
        print('\t'.join(A[:8]))

def merge(BP, sample_order, v_id, use_product):
    if len(BP) == 1:
        A = BP[0].l.rstrip().split('\t')
        #tack on id to SNAME
        s_start=A[7].find('SNAME=')
        s_end=A[7].find(';',s_start)
        if (s_end > -1):
            A[7] = A[7][:s_start] + \
                    A[7][s_start:s_end] + \
                    ':' + A[2] + \
                    A[7][s_end:]
        else:
            A[7]+= ':' + A[2]

        # reset the id to be unique in this file
        v_id += 1
        A[2] = str(v_id)

        #clip out old mate id
        s_start=A[7].find('MATEID=')
        s_end=A[7].find(';',s_start)
        if (s_end > -1):
            A[7] = A[7][:s_start] + A[7][s_end+1:]
        elif (s_start > -1):
            A[7] = A[7][:s_start]

        #clip out old event id
        s_start=A[7].find('EVENT=')
        s_end=A[7].find(';', s_start)
        if (s_end > -1):
            A[7] = A[7][:s_start] + A[7][s_end+1:]
        elif (s_start > -1):
            A[7] = A[7][:s_start]

        #add new mate
        A[7]+= ';EVENT=' + A[2]

        #add new alg
        if use_product:
            A[7]+= ';ALG=PROD'
        else:
            A[7] += ';ALG=SUM'

        print_var_line('\t'.join(A))
        return v_id

    #Sweep the set.  Find the largest intersecting set.  Remove it.  Continue.
    import heapq

    BP.sort(key=lambda x: x.start_l)

    BP_i = range(len(BP))
    C = []

    while len(BP_i) > 0:
        h_l = []
        max_c = []
        max_c_len = 0
        for i in BP_i:
            while (len(h_l) > 0) and (h_l[0][0] < BP[i].start_l):
                heapq.heappop(h_l)

            heapq.heappush(h_l, (BP[i].end_l, i))

            # at this point everything in h_l intersects on the left
            # but we need to take into account what is going on on the right
            h_r = []
            h_l_i = [x[1] for x in h_l]
            h_l_i.sort(key=lambda x:BP[x].start_r)
            for j in h_l_i:
                while (len(h_r) > 0) and (h_r[0][0] < BP[j].start_r):
                    heapq.heappop(h_r)

                heapq.heappush(h_r, (BP[j].end_r, j))

                if max_c_len < len(h_r):
                    max_c_len = len(h_r)
                    max_c = [y[1] for y in h_r]

        C.append(max_c)
        for c in max_c:
            BP_i.remove(c)

    for c in C:
        L = []
        R = []
        for b_i in c:
            b = BP[b_i]
            L.append([b.start_l,b.end_l,b.p_l])
            R.append([b.start_r,b.end_r,b.p_r])

        [start_R, end_R, a_R] = l_bp.align_intervals(R)
        [start_L, end_L, a_L] = l_bp.align_intervals(L)

        p_L = [0] * len(a_L[0])
        p_R = [0] * len(a_R[0])

        for c_i in range(len(c)):
            for i in range(len(a_L[c_i])):
                p_L[i] += a_L[c_i][i]

            for i in range(len(a_R[c_i])):
                p_R[i] += a_R[c_i][i]

        ALG = 'SUM'
        if use_product:
            pmax_i_L = p_L.index(max(p_L))
            pmax_i_R = p_R.index(max(p_R))

            miss = 0
            for c_i in range(len(c)):
                if (a_L[c_i][pmax_i_L] == 0) or (a_R[c_i][pmax_i_R] == 0):
                    miss += 1
            if miss == 0:
                ALG = "PROD"
                ls_p_L = [get_ls(1)] * len(a_L[0])
                ls_p_R = [get_ls(1)] * len(a_R[0])
                for c_i in range(len(c)):
                    for i in range(len(a_L[c_i])):
                        ls_p_L[i] = ls_multiply(ls_p_L[i], get_ls(a_L[c_i][i]))

                    for i in range(len(a_R[c_i])):
                        ls_p_R[i] = ls_multiply(ls_p_R[i], get_ls(a_R[c_i][i]))

                ls_sum_L = get_ls(0)
                ls_sum_R = get_ls(0)

                for ls_p in ls_p_L:
                    ls_sum_L = ls_add(ls_sum_L, ls_p)

                for ls_p in ls_p_R:
                    ls_sum_R = ls_add(ls_sum_R, ls_p)

                p_L = []
                for ls_p in ls_p_L:
                    p_L.append(get_p(ls_divide(ls_p, ls_sum_L)))

                p_R = []
                for ls_p in ls_p_R:
                    p_R.append(get_p(ls_divide(ls_p, ls_sum_R)))

        sum_L = sum(p_L)
        sum_R = sum(p_R)
        p_L = [x/sum_L for x in p_L]
        p_R = [x/sum_L for x in p_R]

        [clip_start_L, clip_end_L] = l_bp.trim(p_L)
        [clip_start_R, clip_end_R] = l_bp.trim(p_R)

        new_start_L = start_L + clip_start_L
        new_end_L = end_L - clip_end_L

        new_start_R = start_R + clip_start_R
        new_end_R = end_R - clip_end_R

        p_L = p_L[clip_start_L:len(p_L)-clip_end_L]
        p_R = p_R[clip_start_R:len(p_R)-clip_end_R]

        s_p_L = sum(p_L)
        s_p_R = sum(p_R)

        p_L = [x/s_p_L for x in p_L]
        p_R = [x/s_p_R for x in p_R]

        max_i_L = p_L.index(max(p_L))
        max_i_R = p_R.index(max(p_R))

        ninefive_i_L_start = max_i_L
        ninefive_i_L_end = max_i_L
        ninefive_i_L_total = p_L[max_i_L]
        updated = 0
        while (ninefive_i_L_total < 0.95):
            if (ninefive_i_L_start <= 0) and (ninefive_i_L_end >= (len(p_L)-1)):
                break
            ninefive_i_L_start = max(0, ninefive_i_L_start - 1)
            ninefive_i_L_end = min(len(p_L)-1, ninefive_i_L_end +1)
            ninefive_i_L_total = sum(p_L[ninefive_i_L_start:ninefive_i_L_end+1])
        ninefive_i_L_start = ninefive_i_L_start - max_i_L
        ninefive_i_L_end = ninefive_i_L_end - max_i_L

        ninefive_i_R_start = max_i_R
        ninefive_i_R_end = max_i_R
        ninefive_i_R_total = p_R[max_i_R]
        updated = 0
        while (ninefive_i_R_total < 0.95):
            if (ninefive_i_R_start <= 0) and (ninefive_i_R_end >= len(p_R)-1):
                break
            ninefive_i_R_start = max(0, ninefive_i_R_start - 1)
            ninefive_i_R_end = min(len(p_R)-1, ninefive_i_R_end +1)
            ninefive_i_R_total = sum(p_R[ninefive_i_R_start:ninefive_i_R_end+1])
        ninefive_i_R_end = ninefive_i_R_end - max_i_R
        ninefive_i_R_start = ninefive_i_R_start - max_i_R
        CIPOS95=str(ninefive_i_L_start) + ',' + str(ninefive_i_L_end)
        CIEND95=str(ninefive_i_R_start) + ',' + str(ninefive_i_R_end)

        CHROM = BP[c[0]].chr_l
        POS = new_start_L + max_i_L
        v_id += 1
        ID = str(v_id)
        REF = 'N'

        ALT = ''
        if BP[c[0]].sv_type == 'BND':
            if BP[c[0]].strands[:2] == '++':
                ALT = 'N]' + \
                        BP[c[0]].chr_r + \
                        ':' + \
                        str(new_start_R + max_i_R) + \
                        ']'
            elif BP[c[0]].strands[:2] == '-+':
                ALT = ']' + \
                        BP[c[0]].chr_r + \
                        ':' + \
                        str(new_start_R + max_i_R) + \
                        ']N'
            elif BP[c[0]].strands[:2] == '+-':
                ALT = 'N[' + \
                        BP[c[0]].chr_r + \
                        ':' + \
                        str(new_start_R + max_i_R) + \
                        '['
            elif BP[c[0]].strands[:2] == '--':
                ALT = '[' + \
                        BP[c[0]].chr_r + \
                        ':' + \
                        str(new_start_R + max_i_R) + \
                        '[N'

        else:
            ALT = '<' + BP[c[0]].sv_type + '>'
        QUAL = 0.0
        FILTER = '.'
        FORMAT = BP[c[0]].l.split('\t')[8]
        SVTYPE = BP[c[0]].sv_type

        STRANDS = ''
        strand_map = {}
        e_type_map = {}

        SU = 0
        PE = 0
        SR = 0

        s_name_list = []

        gt_list = []

        for b_i in c:
            A = BP[b_i].l.rstrip().split('\t')
            if A[5].isdigit():
                QUAL += float(A[5])

            m = l_bp.to_map(A[7])

            for strand_entry in m['STRANDS'].split(','):
                s_type,s_count = strand_entry.split(':')
                if s_type not in strand_map:
                    strand_map[s_type] = 0
                strand_map[s_type] += int(s_count)

            SU += int(m['SU'])
            PE += int(m['PE'])
            SR += int(m['SR'])

            s_name_list.append(m['SNAME'] + ':' + A[2])

            gt_list += A[9:]

        SNAME=','.join(s_name_list)

        GTS = '\t'.join(gt_list)

        strand_types_counts = []
        for strand in strand_map:
            strand_types_counts.append(strand + ':' + str(strand_map[strand]))
        STRANDS = ','.join(strand_types_counts)

        if SVTYPE=='DEL':
            SVLEN = (new_start_L + max_i_L) - (new_start_R + max_i_R)
        else:
            SVLEN = (new_start_R + max_i_R) - (new_start_L + max_i_L)

        # Don't set SVLEN if we have an interchromosomal event. Doesn't make any sense.
        if BP[c[0]].chr_l != BP[c[0]].chr_r:
            SVLEN = None

        END = new_start_R + max_i_R
        CIPOS=','.join([str(x) for x in [-1*max_i_L, len(p_L) - max_i_L - 1]])
        CIEND=','.join([str(x) for x in [-1*max_i_R, len(p_R) - max_i_R - 1]])
        IMPRECISE='IMPRECISE'
        PRPOS=','.join([str(x) for x in p_L])
        PREND=','.join([str(x) for x in p_R])


        if (int(CIPOS.split(',')[0]) > int(CIPOS95.split(',')[0])) or \
            (int(CIPOS.split(',')[1]) < int(CIPOS95.split(',')[1])) or \
            (int(CIEND.split(',')[0]) > int(CIEND95.split(',')[0])) or \
            (int(CIEND.split(',')[1]) < int(CIEND95.split(',')[1])):
            sys.stderr.write(CIPOS + "\t" + str(CIPOS95) + "\n")
            sys.stderr.write(CIEND + "\t" + str(CIEND95) + "\n")

        I = ['SVTYPE='   + str(SVTYPE),
             'STRANDS='  + str(STRANDS)
            ]
        if SVLEN:
            I += ['SVLEN='    + str(SVLEN)]
        I += ['CIPOS='    + str(CIPOS),
             'CIEND='    + str(CIEND),
             'CIPOS95='  + str(CIPOS95),
             'CIEND95='  + str(CIEND95),
                           str(IMPRECISE),
             'SU='       + str(SU),
             'PE='       + str(PE),
             'SR='       + str(SR),
             'PRPOS='    + str(PRPOS),
             'PREND='    + str(PREND),
             'ALG='      + str(ALG),
             'SNAME='    + str(SNAME)]

        if BP[c[0]].sv_type == 'BND':
            I.append('EVENT=' + str(ID))
        else:
            I.append('END=' + str(END))

        INFO = ';'.join(I)

        QUAL = str(QUAL)

        O = [CHROM,POS,ID,REF,ALT,QUAL,FILTER,INFO]

        print_var_line('\t'.join([str(o) for o in O]))
    return v_id

def r_cluster(BP_l, sample_order, v_id, use_product):
    # need to resort based on the right side, then extract clusters
    BP_l.sort(key=lambda x: x.start_r)
    BP_l.sort(key=lambda x: x.chr_r)

    BP_r = []
    BP_max_end_r = -1
    BP_chr_r = ''

    for b in BP_l:
        if (len(BP_r) == 0) or \
           ((b.start_r <= BP_max_end_r) and \
           (b.chr_r == BP_chr_r)):
            BP_r.append(b)
            BP_max_end_r = max(BP_max_end_r, b.end_r)
            BP_chr_r = b.chr_r
        else:
            v_id = merge(BP_r, sample_order, v_id, use_product)
            BP_r = [b]
            BP_max_end_r = b.end_r
            BP_chr_r = b.chr_r

    if len(BP_r) > 0:
        v_id = merge(BP_r, sample_order, v_id, use_product)

    return v_id

def l_cluster_by_line(file_name, percent_slop=0, fixed_slop=0, use_product=False):
    v_id = 0
    vcf_lines = []
    vcf_headers = list()
    infile=open(file_name, 'r')

    header = ''
    samples = ''

    for l in infile:
      if l[0] == '#':
        if l[1] != '#':
          samples = l.rstrip().split('\t')[9:]
        else:
          # ignore fileDate
          if l[:10] == '##fileDate':
            continue
          if l not in vcf_headers:
            vcf_headers.append(l)
      if l[0] != '#':
        break

    vcf_headers.append("#CHROM\tPOS\tID\tREF\tALT\tQUAL\tFILTER\tINFO\n")

    sample_order = []
    for header in vcf_headers:
      if header[:8] == '##SAMPLE':
        sample_order.append(header.rstrip()[13:-1])
      print(header),

    BP_l = []
    BP_sv_type = ''
    BP_max_end_l = -1
    BP_chr_l = ''

    b=l_bp.breakpoint(l_bp.parse_vcf_record(l), percent_slop=percent_slop, fixed_slop=fixed_slop)
    BP_l.append(b)
    BP_max_end_l = max(BP_max_end_l, b.end_l)
    BP_chr_l = b.chr_l
    BP_sv_type = b.sv_type

    for l in infile:
      b=l_bp.breakpoint(l_bp.parse_vcf_record(l), percent_slop=percent_slop, fixed_slop=fixed_slop)
      if (len(BP_l) == 0) or ((b.start_l <= BP_max_end_l) and (b.chr_l == BP_chr_l) and (b.sv_type == BP_sv_type)):
        BP_l.append(b)
        BP_max_end_l = max(BP_max_end_l, b.end_l)
        BP_chr_l = b.chr_l
        BP_sv_type = b.sv_type
      else:
        v_id = r_cluster(BP_l, sample_order, v_id, use_product)
        BP_l = [b]
        BP_max_end_l = b.end_l
        BP_sv_type = b.sv_type
        BP_chr_l = b.chr_l

    if len(BP_l) > 0:
        v_id = r_cluster(BP_l, sample_order, v_id, use_product)

    infile.close()

def description():
    return 'merge LUMPY calls inside a single file from svtools lsort'

def epilog():
    return 'Note that if both slop parameters are set then the maximum is used.'

def add_arguments_to_parser(parser):
    parser.add_argument('-i', '--inFile', metavar='<FILE>', help='a sorted LUMPY output file generated by lsort. Column 7 must have the format sample:variantID')
    parser.add_argument('-p', '--percent-slop', metavar='<FLOAT>', type=float, default=0.0, help='increase the the breakpoint confidence interval both up and down stream by a given proportion of the original size')
    parser.add_argument('-f', '--fixed-slop', metavar='<INT>', type=int, default=0, help='increase the the breakpoint confidence interval both up and down stream by a given fixed size')
    parser.add_argument('--product', dest='use_product', action='store_true', default=False, help='calculate breakpoint PDF and position using product')
    parser.set_defaults(entry_point=run_from_args)

def command_parser():
    parser = argparse.ArgumentParser(description=description(), epilog=epilog())
    add_arguments_to_parser(parser)
    return parser

def run_from_args(args):
    l_cluster_by_line(args.inFile,
            percent_slop=args.percent_slop,
            fixed_slop=args.fixed_slop,
            use_product=args.use_product)

if __name__ == "__main__":
    parser = command_parser()
    args = parser.parse_args()
    sys.exit(args.entry_point(args))



import argparse
import sys
import time

import svtools_star.svtools.vcf.file
import svtools_star.svtools.vcf.variant
import svtools_star.svtools.utils as su
from svtools_star.svtools.vcftobedpeconverter import VcfToBedpeConverter

def vcfToBedpe(vcf_file, bedpe_out):
    converter = VcfToBedpeConverter()
    vcf = svtools.vcf.file.Vcf()
    in_header = True
    header = []
    sample_list = []
    bnds = dict()
    sec_bnds = dict()
    v = []
    for line in vcf_file:
        if in_header:
            if line[0:2] == '##':
                if line.split('=')[0] == '##fileformat':
                    line = '##fileformat=' + "BEDPE" + '\n'
                if line.split('=')[0] == '##fileDate':
                    line = '##fileDate=' + time.strftime('%Y%m%d') + '\n'
                header.append(line)
                continue
            elif line[0] == '#' and line[1] != '#':
                sample_list = line.rstrip().split('\t')[9:]
                header.append(line)
                continue
            else:
                # print header
                in_header = False
                vcf.add_header(header)
                if "SVTYPE" in [info.id for info in vcf.info_list]:
                   vcf.add_info_after("SVTYPE", "POS", 1, 'Integer', 'Position of the variant described in this record')
                header=vcf.get_header()
                bedpe_out.write(header[:header.rfind('\n')] + '\n')
                final_header_line = ['#CHROM_A',
                        'START_A',
                        'END_A',
                        'CHROM_B',
                        'START_B',
                        'END_B',
                        'ID',
                        'QUAL',
                        'STRAND_A',
                        'STRAND_B',
                        'TYPE',
                        'FILTER',
                        'NAME_A',
                        'REF_A',
                        'ALT_A',
                        'NAME_B',
                        'REF_B',
                        'ALT_B',
                        'INFO_A',
                        'INFO_B']

                if len(sample_list) > 0:
                    bedpe_out.write('\t'.join(final_header_line + ['FORMAT','\t'.join(map(str,sample_list))]) + '\n')
                else:
                    bedpe_out.write('\t'.join(final_header_line) + '\n')

        v = line.rstrip().split('\t')
        var = svtools.vcf.variant.Variant(v, vcf)
        var.set_info("POS", var.pos)
        unique_name = var.var_id
        if 'EVENT' in var.info:
            unique_name = var.info['EVENT']
        if var.info['SVTYPE'] != 'BND':
            bedpe_out.write(str(converter.convert(var)) + '\n')
        else:
            if 'SECONDARY' in var.info:
                if unique_name in bnds:
                    #primary
                    var1 = bnds[unique_name]
                    bedpe_out.write(str(converter.convert(var1, var)) + '\n')
                    del bnds[unique_name]
                else:
                    sec_bnds.update({unique_name:var})
            else:
                bnds.update({unique_name:var})
                continue
    intersected_keys = bnds.viewkeys() & sec_bnds.viewkeys()
    for key in intersected_keys:
        bedpe_out.write(str(converter.convert(bnds[key], sec_bnds[key])) + '\n')
        del bnds[key]
        del sec_bnds[key]
    if bnds is not None:
        for bnd in bnds:
            sys.stderr.write('Warning: missing secondary multiline variant at ID:' + bnd + '\n')
            bedpe_out.write(str(converter.convert(bnds[bnd], None)) + '\n')
    if sec_bnds is not None:
        for bnd in sec_bnds:
            sys.stderr.write('Warning: missing primary multiline variant at ID:' + bnd + '\n')
            bedpe_out.write(str(converter.convert(None, sec_bnds[bnd])) + '\n')

    # close the files
    bedpe_out.close()
    return

def description():
    return 'convert a VCF file to a BEDPE file'

def epilog():
    return 'The input VCF file can be gzipped if it is specified explicitly.'

def add_arguments_to_parser(parser):
    parser.add_argument('-i', '--input', metavar='<VCF>', default=None, help='VCF input (default: stdin)')
    parser.add_argument('-o', '--output', metavar='<BEDPE>', type=argparse.FileType('w'), default=sys.stdout, help='output BEDPE to write (default: stdout)')
    parser.set_defaults(entry_point=run_from_args)

def command_parser():
    parser = argparse.ArgumentParser(description=description(), epilog=epilog())
    add_arguments_to_parser(parser)
    return parser

def run_from_args(args):
    with su.InputStream(args.input) as stream:
        return vcfToBedpe(stream, args.output)

# initialize the script
if __name__ == '__main__':
    parser = command_parser()
    args = parser.parse_args()
    sys.exit(args.entry_point(args))



import sys
import argparse
from svtools.external_cmd import ExternalCmd

class VcfSort(ExternalCmd):
    def __init__(self):
        super(VcfSort, self).__init__('vcfsort', 'bin/vcfsort')

def description():
    return 'sort a VCF file'

def add_arguments_to_parser(parser):
    parser.add_argument('input', metavar='<VCF>', nargs='?', help='VCF file to sort (default: stdin)')
    parser.add_argument('output', metavar='<VCF>', nargs='?', help='output file to write to (default: stdout)')
    parser.set_defaults(entry_point=run_from_args)

def command_parser():
    parser = argparse.ArgumentParser(description=description())
    add_arguments_to_parser(parser)
    return parser

def run_from_args(args):
    opts = list()
    if args.input:
        opts.append(args.input)
    if args.output:
        opts.append(args.output)

    sort_cmd_runner = VcfSort()
    sort_cmd_runner.run_cmd_with_options(opts)

if __name__ == "__main__":
    parser = command_parser()
    args = parser.parse_args()
    sys.exit(args.entry_point(args))



import sys
import argparse
from svtools.external_cmd import ExternalCmd

class BedpeSort(ExternalCmd):
    def __init__(self):
        super(BedpeSort, self).__init__('bedpesort', 'bin/bedpesort')

def description():
    return 'sort a BEDPE file'

def add_arguments_to_parser(parser):
    parser.add_argument('input', metavar='<BEDPE file>', nargs='?', help='BEDPE file to sort')
    parser.add_argument('output', metavar='<output file>', nargs='?', help='output file to write to')
    parser.set_defaults(entry_point=run_from_args)

def command_parser():
    parser = argparse.ArgumentParser(description=description())
    add_arguments_to_parser(parser)
    return parser

def run_from_args(args):
    opts = list()
    if args.input:
        opts.append(args.input)
    if args.output:
        opts.append(args.output)

    sort_cmd_runner = BedpeSort()
    sort_cmd_runner.run_cmd_with_options(opts)

if __name__ == "__main__":
    parser = command_parser()
    args = parser.parse_args()
    sys.exit(args.entry_point(args))



from svtools_star.svtools.vcf.variant import Variant
import re

class BedpeToVcfConverter(object):
    '''
    This is a class to take Bedpe objects and convert them to VCF variant object(s)
    '''

    def __init__(self, vcf):
        '''
        Initialize a new converter with a reference to Vcf object
        '''
        self.vcf_header = vcf

    @staticmethod
    def adjust_by_tag(bedpe, info_tag, strand, coordinate):
        '''
        Undo adjustments to BEDPE coordinates based on strand and INFO tag values
        '''
        if strand == '-' and bedpe.svtype == 'BND':
            coordinate += 1
        if info_tag in bedpe.info:
            value_array = re.split('=|,', ''.join(filter(lambda x: info_tag in x, bedpe.info.split(';'))))
            coordinate -= int(value_array[1])
        return coordinate

    @staticmethod
    def determine_sep(o2):
        '''
        Given the orientation of the other breakend, determine separator for VCF alt
        '''
        if o2 == '+':
            return ']'
        else:
            return '['

    @staticmethod
    def determine_flanks(o1):
        '''
        Given the orientation of the breakend, return proper flanking sequence strings
        '''
        if o1 == '+':
            return ('N', '')
        else:
            return ('', 'N')

    def bnd_alt_string(self, o1, o2, chrom, pos):
        '''
        Given a Bedpe object generate the correct alt string for a BND VCF entry
        '''
        alt_string = '{3}{0}{1}:{2}{0}{4}'
        return alt_string.format(self.determine_sep(o2), chrom, pos, *self.determine_flanks(o1))

    def convert(self, bedpe):
        '''
        Convert a bedpe object to Vcf object(s). Returns a list of entries.
        '''
        adjust_tag1, adjust_tag2 = 'CIPOS', 'CIEND'
        if bedpe.malformedFlag == 1:
            adjust_tag2, adjust_tag1 = adjust_tag1, adjust_tag2

        b1 = self.adjust_by_tag(bedpe, adjust_tag1, bedpe.o1, bedpe.s1)
        primary_bedpe_list = [
                bedpe.c1,
                b1,
                bedpe.orig_name1,
                bedpe.orig_ref1,
                bedpe.orig_alt1,
                bedpe.score,
                bedpe.filter,
                bedpe.info1
                ] + bedpe.misc
        var = Variant(primary_bedpe_list, self.vcf_header)
        to_return = [var]

        if bedpe.svtype == 'BND':
            b2 = self.adjust_by_tag(bedpe, adjust_tag2, bedpe.o2, bedpe.s2)

            secondary_bedpe_list = [
                    bedpe.c2,
                    b2,
                    bedpe.orig_name2,
                    bedpe.orig_ref2,
                    bedpe.orig_alt2,
                    bedpe.score,
                    bedpe.filter,
                    bedpe.info2
                    ] + bedpe.misc

            var2 = Variant(secondary_bedpe_list, self.vcf_header)
            if bedpe.malformedFlag == 0:
                to_return += [var2]
            elif bedpe.malformedFlag == 1:
                #Only returning one of our entries
                to_return[0] = var2

        return to_return




import argparse, sys
import svtools.lsort
import svtools.lmerge
import svtools.vcfpaste
import svtools.copynumber
import svtools.afreq
import svtools.bedpetobed12
import svtools.bedpetovcf
import svtools.vcftobedpe
import svtools.vcfsort
import svtools.bedpesort
import svtools.genotype
import svtools.prune
import svtools.varlookup

class SupportAction(argparse.Action):
    def __call__(self, parser, namespace, values, option_string=None):
        support_string = 'For further help or to report a bug, please open an issue on the svtools repository: https://github.com/hall-lab/svtools/issues'
        print(support_string)
        sys.exit()

def svtools_cli_parser():
    parser = argparse.ArgumentParser(description='Comprehensive utilities to explore structural variation in genomes', prog='svtools')
    version_string = '%(prog)s {0}'.format(svtools.__version__)
    parser.add_argument('--version', action='version', version=version_string)
    parser.add_argument('--support', action=SupportAction, nargs=0, help='information on obtaining support')
    subparsers = parser.add_subparsers(title=None, metavar='subcommand', help='description')

    lsort = subparsers.add_parser('lsort', help=svtools.lsort.description(), epilog=svtools.lsort.epilog())
    svtools.lsort.add_arguments_to_parser(lsort)

    lmerge = subparsers.add_parser('lmerge', help=svtools.lmerge.description(), epilog=svtools.lmerge.epilog())
    svtools.lmerge.add_arguments_to_parser(lmerge)

    vcf_paste = subparsers.add_parser('vcfpaste', help=svtools.vcfpaste.description(), epilog=svtools.vcfpaste.epilog())
    svtools.vcfpaste.add_arguments_to_parser(vcf_paste)

    copynumber = subparsers.add_parser('copynumber', help=svtools.copynumber.description(), epilog=svtools.copynumber.epilog())
    svtools.copynumber.add_arguments_to_parser(copynumber)

    genotype = subparsers.add_parser('genotype', help=svtools.genotype.description())
    svtools.genotype.add_arguments_to_parser(genotype)

    afreq = subparsers.add_parser('afreq', help=svtools.afreq.description(), epilog=svtools.afreq.epilog())
    svtools.afreq.add_arguments_to_parser(afreq)

    bedpetobed12 = subparsers.add_parser('bedpetobed12', help=svtools.bedpetobed12.description(), epilog=svtools.bedpetobed12.epilog())
    svtools.bedpetobed12.add_arguments_to_parser(bedpetobed12)

    bedpetovcf = subparsers.add_parser('bedpetovcf', help=svtools.bedpetovcf.description(), epilog=svtools.bedpetovcf.epilog())
    svtools.bedpetovcf.add_arguments_to_parser(bedpetovcf)

    vcftobedpe = subparsers.add_parser('vcftobedpe', help=svtools.vcftobedpe.description(), epilog=svtools.vcftobedpe.epilog())
    svtools.vcftobedpe.add_arguments_to_parser(vcftobedpe)

    vcfsort = subparsers.add_parser('vcfsort', help=svtools.vcfsort.description())
    svtools.vcfsort.add_arguments_to_parser(vcfsort)

    bedpesort = subparsers.add_parser('bedpesort', help=svtools.bedpesort.description())
    svtools.bedpesort.add_arguments_to_parser(bedpesort)

    prune = subparsers.add_parser('prune', help=svtools.prune.description(), epilog=svtools.prune.epilog())
    svtools.prune.add_arguments_to_parser(prune)

    varlookup = subparsers.add_parser('varlookup', help=svtools.varlookup.description())
    svtools.varlookup.add_arguments_to_parser(varlookup)

    return parser

def main():
    parser = svtools_cli_parser()
    args = parser.parse_args()
    sys.exit(args.entry_point(args))

if __name__ == '__main__':
    main()




import argparse, sys
from svtools.vcf.file import Vcf
from svtools.vcf.variant import Variant
import svtools.utils as su

class UpdateInfo(object):
    def __init__(self, vcf_stream):
        self.vcf_stream = vcf_stream

    @staticmethod
    def numeric_alleles(gt_string):
        '''
        Convert a VCF allele string into integers
        '''
        gt = gt_string.split('/')
        if len(gt) == 1:
            gt = gt_string.split('|')
        return map(int, gt)

    def execute(self, output_handle=sys.stdout):
        in_header = True
        header = []
        vcf = Vcf()
        vcf_out = output_handle

        # read input VCF
        for line in self.vcf_stream:
            if in_header:
                if line.startswith('##'):
                    header.append(line) 
                    continue
                elif line.startswith('#CHROM'):
                    v = line.rstrip().split('\t')
                    header.append('\t'.join(v))

                    in_header = False
                    vcf.add_header(header)
                    
                    vcf.add_info('AF', 'A', 'Float', 'Allele Frequency, for each ALT allele, in the same order as listed')
                    vcf.add_info('NSAMP', '1', 'Integer', 'Number of samples with non-reference genotypes')
                    vcf.add_info('MSQ', '1', 'Float', 'Mean sample quality of positively genotyped samples')

                    # write header
                    vcf_out.write(vcf.get_header() + '\n')
                    #vcf_out.write('\t' + '\t'.join(v[8:]) + '\n')
                continue

            v = line.rstrip().split('\t')
            var = Variant(v, vcf)

            # extract genotypes from VCF
            num_alt = len(var.alt.split(','))
            alleles = [0] * (num_alt + 1)
            num_samp = 0
            sum_sq = 0.0

            for gt in var.genotypes():
                gt_string = gt.get_format('GT')

                if '.' not in gt_string:
                    indexes = self.numeric_alleles(gt_string)

                    for i in indexes:
                        alleles[i] += 1

                    # iterate the number of non-reference samples
                    if sum(indexes) > 0:
                        num_samp += 1
                        try:
                            sum_sq += float(gt.get_format('SQ'))
                        except KeyError:
                            pass

            allele_sum = float(sum(alleles))
            allele_freq = ['.'] * len(alleles)

            # populate AF
            if allele_sum > 0:
                for i in xrange(len(alleles)):
                    allele_freq[i] = alleles[i] / allele_sum
                var.info['AF'] = ','.join(map(str, ['%.4g' % a for a in allele_freq[1:]]))
            else:
                var.info['AF'] = ','.join(map(str, allele_freq[1:]))
            
            # populate NSAMP
            var.info['NSAMP'] = num_samp
            if num_samp > 0:
                msq = '%0.2f' % (sum_sq / num_samp)
            else:
                msq = '.'
            var.info['MSQ'] = msq

            # after all samples have been processed, write
            vcf_out.write(var.get_var_string(use_cached_gt_string=True) + '\n')
        vcf_out.close()

def description():
    return 'add allele frequency information to a VCF file'

def epilog():
    return 'Specify the path to an (optionally) bgzipped VCF. If no file is specified then input is read from stdin.'


def add_arguments_to_parser(parser):
    parser.add_argument(metavar='<VCF>', dest='input_vcf', nargs='?', default=None, help='VCF input')
    parser.set_defaults(entry_point=run_from_args)

def command_parser():
    parser = argparse.ArgumentParser(description=description(), epilog=epilog())
    add_arguments_to_parser(parser)
    return parser

def run_from_args(args):
    with su.InputStream(args.input_vcf) as input_stream:
        updater = UpdateInfo(input_stream)
        updater.execute()

if __name__ == '__main__':
    parser = command_parser()
    args = parser.parse_args()
    sys.exit(args.entry_point(args))



import unittest
import os
import sys
sys.path.insert(0, '../')
import starseqr_utils as su


path = os.path.dirname(__file__)
if path != '':
    os.chdir(path)


class PrimerTestCase(unittest.TestCase):
    '''Tests primer design'''
    def test_runp3_v1(self):
        """test_runp3 with : in middle"""
        seq = "CATCAGCAGAATTCCCGCCATAGGTTAACTCTTTGACATTCTTATTACCACTGTGGGTCTCTTTGGGATCCCAGGGCACTGTAGGCAACTAGAACAATGTCCTTGGACTTGCAGAACTCCAGGAGTTTGCTCTGGTTGAGGTAAGGGTGACATTCCACCTGTAA:TTTTTCAAGACCAAAAGGTGTCACAATATATGAGCCAATAACCACATGCAAATTTAAACTGGAAAATAGAAAAGAAGATAAAGAAGGACATTTTCTATGGAATTCTCCTTTCTGGTGCCTAAGGAGCTCGGCCAAACAACCTTCAAGAGTTTCAGGAGGTGGTGATTAT"
        primer_res = su.run_primer3.runp3('test1', seq)
        assert(primer_res == ('TGGACTTGCAGAACTCCAGG', 'TGCATGTGGTTATTGGCTCA'))

    def test_runp3_v1_1(self):
        """same as v1 in middle but this time use the location"""
        seq = "CATCAGCAGAATTCCCGCCATAGGTTAACTCTTTGACATTCTTATTACCACTGTGGGTCTCTTTGGGATCCCAGGGCACTGTAGGCAACTAGAACAATGTCCTTGGACTTGCAGAACTCCAGGAGTTTGCTCTGGTTGAGGTAAGGGTGACATTCCACCTGTAATTTTTCAAGACCAAAAGGTGTCACAATATATGAGCCAATAACCACATGCAAATTTAAACTGGAAAATAGAAAAGAAGATAAAGAAGGACATTTTCTATGGAATTCTCCTTTCTGGTGCCTAAGGAGCTCGGCCAAACAACCTTCAAGAGTTTCAGGAGGTGGTGATTAT"
        primer_res = su.run_primer3.runp3('test1', seq, 164)
        assert(primer_res == ('TGGACTTGCAGAACTCCAGG', 'TGCATGTGGTTATTGGCTCA'))


    def test_runp3_v2(self):
        """test_runp3 with no target"""
        seq = ":CATCAGCAGAATTCCCGCCATAGGTTAACTCTTTGACATTCTTATTACCACTGTGGGTCTCTTTGGGATCCCAGGGCACTGTAGGCAACTAGAACAATGTCCTTGGACTTGCAGAACTCCAGGAGTTTGCTCTGGTTGAGGTAAGGGTGACATTCCACCTGTAATTTTTCAAGACCAAAAGGTGTCACAATATATGAGCCAATAACCACATGCAAATTTAAACTGGAAAATAGAAAAGAAGATAAAGAAGGACATTTTCTATGGAATTCTCCTTTCTGGTGCCTAAGGAGCTCGGCCAAACAACCTTCAAGAGTTTCAGGAGGTGGTGATTAT"
        primer_res = su.run_primer3.runp3('test1', seq)
        assert(primer_res == ())

    def test_runp3_v3(self):
        """test_runp3 with short seq"""
        seq = "CATCAGCAGAATTCCCGCCATAG"
        primer_res = su.run_primer3.runp3('test1', seq)
        assert(primer_res == ())

    def test_runp3_v4(self):
        """test_runp3 with small target"""
        seq = "CATCAGCA:GAATTCCCGCCATAGGTTAACTCTTTGACATTCTTATTACCACTGTGGGTCTCTTTGGGATCCCAGGGCACTGTAGGCAACTAGAACAATGTCCTTGGACTTGCAGAACTCCAGGAGTTTGCTCTGGTTGAGGTAAGGGTGACATTCCACCTGTAATTTTTCAAGACCAAAAGGTGTCACAATATATGAGCCAATAACCACATGCAAATTTAAACTGGAAAATAGAAAAGAAGATAAAGAAGGACATTTTCTATGGAATTCTCCTTTCTGGTGCCTAAGGAGCTCGGCCAAACAACCTTCAAGAGTTTCAGGAGGTGGTGATTAT"
        primer_res = su.run_primer3.runp3('test1', seq)
        assert(primer_res == ())

    def test_runp3_v5(self):
        """test_runp3 with no target"""
        seq = "CATCAGCAGAATTCCCGCCATAGGTTAACTCTTTGACATTCTTATTACCACTGTGGGTCTCTTTGGGATCCCAGGGCACTGTAGGCAACTAGAACAATGTCCTTGGACTTGCAGAACTCCAGGAGTTTGCTCTGGTTGAGGTAAGGGTGACATTCCACCTGTAATTTTTCAAGACCAAAAGGTGTCACAATATATGAGCCAATAACCACATGCAAATTTAAACTGGAAAATAGAAAAGAAGATAAAGAAGGACATTTTCTATGGAATTCTCCTTTCTGGTGCCTAAGGAGCTCGGCCAAACAACCTTCAAGAGTTTCAGGAGGTGGTGATTAT"
        primer_res = su.run_primer3.runp3('test1', seq)
        assert(primer_res == ('TGGACTTGCAGAACTCCAGG', 'TGCATGTGGTTATTGGCTCA'))

    def test_wrap_runp3_v1(self):
        """test wrapper for runp3"""
        fusions = 'ENST00000372201.4--ENST00000432496.6,ENST00000372201.4--ENST00000394708.6,ENST00000372201.4--ENST00000273980.9,ENST00000372201.4--ENST00000361687.8,ENST00000372201.4--ENST00000394706.7'
        res1 = su.run_primer3.wrap_runp3('chr1:45268528:+:chr4:107152937:-:0:2', fusions)
        assert(res1 == ('CCTGAGGCGGATGTATGGTC', 'ACATACATGCATAAGCCAAGGC'))

    def test_wrap_runp3_v2(self):
        """test wrapper for runp3. Empty fusion list"""
        fusions = ''
        res1 = su.run_primer3.wrap_runp3('chr1:45268528:+:chr4:107152937:-:0:2', fusions)
        assert(res1 == ())


if __name__ == '__main__':
    unittest.main()



import unittest
import sys
sys.path.insert(0, '../')
import starseqr_utils as su
import os
import shutil

path = os.path.dirname(__file__)
if path != '':
    os.chdir(path)


class AssemblyTestCase(unittest.TestCase):
    '''Tests assembly'''

    def setUp(self):
        jxn = 'chr1_45268528_pos_chr4_107152937_neg_0_2]'
        clean_jxn = su.common.safe_jxn(jxn)
        jxn_dir = 'support' + '/' + clean_jxn + '/'
        assem_dir = jxn_dir + 'assem_jxn'
        shutil.rmtree(assem_dir, ignore_errors=True)

    def tearDown(self):
        jxn = 'chr1_45268528_pos_chr4_107152937_neg_0_2]'
        clean_jxn = su.common.safe_jxn(jxn)
        jxn_dir = 'support' + '/' + clean_jxn + '/'
        assem_dir = jxn_dir + 'assem_jxn'
        shutil.rmtree(assem_dir, ignore_errors=True)

    def test_assembly_v1(self):
        """test velvet"""
        res1 = su.run_assembly.get_assembly_info('chr1:45268528:+:chr4:107152937:-:0:2', 'velvet')
        expected_res = ('CAGCCCGGTTGGAGCCTCCGGAGCAGAGGAAGAAGACCATCTGTGGCACCCCCAACTATGTGGCTCCAGAAGTGCTGCTGAGACAGGGCCACGGCCCTGAGGCGGATGTATGGTCACTGGGCTGTGTCATGTCTTGACTCACTTTGTGCTCCATTCCTATATCTAAACTTCAATAATGAAGCCTTGGCTTATGCATGTATGTCTGCTTTTATTCCCAAATACCTGTATAA', '214', 'ENST00000372201.4--ENST00000432496.6,ENST00000372201.4--ENST00000394708.6,ENST00000372201.4--ENST00000273980.9,ENST00000372201.4--ENST00000361687.8,ENST00000372201.4--ENST00000394706.7')
        assert(res1 == expected_res)


if __name__ == '__main__':
    unittest.main()



import os
import itertools
from ctypes import *

def load_ssw_library():
    mod_path = os.path.split(__file__)[0]
    search_path = os.path.join(mod_path, "..")
    libname = "_libssw"
    for (root, dirs, files) in os.walk(search_path):
        for fn in files:
            if fn.lower().startswith(libname) and fn.lower().endswith(".so"):
                libpath = os.path.join(root, fn)
                return cdll.LoadLibrary(libpath)
    return cdll.LoadLibrary(libname)

# load our library
libssw = load_ssw_library()

# Types
class AlignmentResult(Structure):
    _fields_ = [
        ("score", c_uint16),
        ("score2", c_uint16),
        ("ref_begin", c_int32),
        ("ref_end", c_int32),
        ('query_begin', c_int32),
        ('query_end', c_int32),
        ('ref_end2', c_int32),
        ('cigar', POINTER(c_uint32)),
        ('cigarLen', c_int32),
    ]

ssw_profile_p = c_void_p
matrix_type = c_int8
symbol_type = c_int8

# ssw_init method
ssw_profile_init = libssw.ssw_init
ssw_profile_init.argtypes = [POINTER(c_int8), c_int32, POINTER(c_int8), c_int32, c_int8]
ssw_profile_init.restype = ssw_profile_p

# init_destroy function
ssw_profile_del = libssw.init_destroy
ssw_profile_del.argtypes =  [ssw_profile_p]
ssw_profile_del.restype = None

# ssw_align function
ssw_align_init = libssw.ssw_align
ssw_align_init.argtypes = [ssw_profile_p, POINTER(c_int8), c_int32, c_uint8, c_uint8, c_uint8, c_uint16, c_int32, c_int32]
ssw_align_init.restype = POINTER(AlignmentResult)

# align_destroy function
ssw_align_del = libssw.align_destroy
ssw_align_del.argtypes = [POINTER(AlignmentResult)]
ssw_align_del.restype = None

# cigar_int_to_len function
cigar_int_to_len = libssw.cigar_int_to_len
cigar_int_to_len.argtypes = [c_int32]
cigar_int_to_len.restype = c_int32

# cigar_int_to_op function
cigar_int_to_op = libssw.cigar_int_to_op
cigar_int_to_op.argtypes = [c_int32]
cigar_int_to_op.restype = c_char

# flags
FLAG_BEST_POS = 0
FLAG_FILTER_SCORE = 1
FLAG_FILTER_DISTANCE = 2
FLAG_BUILD_CIGAR = 3



from svtools_star.svtools.bedpe import Bedpe
import re

class VcfToBedpeConverter(object):
    '''
    This is a class to take Vcf object(s) and convert them to Bedpe lines
    '''

    def __init__(self):
        '''
        Initialize a new converter
        '''
        # NOTE The below is ugly but intended to match things like [2:222[ and capture the brackets
        self.bnd_regex = re.compile(r'([][])(.+?)([][])')

    def parse_bnd_alt_string(self, alt_string):
        '''
        Parse the BND alt string and return separators and region
        '''
        result = self.bnd_regex.findall(alt_string)
        assert result
        sep1, region, sep2 = result[0]
        assert sep1 == sep2
        chrom2, breakpoint2 = region.split(':')
        breakpoint2 = int(breakpoint2)
        return sep1, chrom2, breakpoint2

    def bnd_breakpoints(self, vcf_variant):
        '''
        Return a tuple containing calculated breakpoints and orientations for a BND variant
        '''
        chrom1 = vcf_variant.chrom
        breakpoint1 = vcf_variant.pos
        orientation1 = orientation2 = '+'
        sep, chrom2, breakpoint2 = self.parse_bnd_alt_string(vcf_variant.alt)

        if vcf_variant.alt.startswith(sep):
            orientation1 = '-'
            breakpoint1 -= 1

        if sep == '[':
            orientation2 = '-'
            breakpoint2 -= 1

        return (chrom1,
                breakpoint1,
                breakpoint1,
                chrom2,
                breakpoint2,
                breakpoint2,
                orientation1,
                orientation2)

    @staticmethod
    def simple_breakpoints(vcf_variant):
        '''
        Return a tuple containing breakpoints and orientations for simple SVs
        '''
        breakpoint1 = vcf_variant.pos
        try:
            breakpoint2 = int(vcf_variant.info['END'])
        except KeyError:
            raise ValueError('END entry in VCF required for conversion to BEDPE')

        orientation1 = '+'
        orientation2 = '-'

        if 'STRANDS' in vcf_variant.info:
            strands = vcf_variant.info['STRANDS']
            orientation1, orientation2 = strands[:2]

        return (vcf_variant.chrom,
            breakpoint1,
            breakpoint1,
            vcf_variant.chrom,
            breakpoint2,
            breakpoint2,
            orientation1,
            orientation2)

    @staticmethod
    def adjust_coordinate(vcf_variant, info_tag, start, end):
        '''
        Return adjusted start and end coordinates according to the contents
        of the tag (if it exists)
        '''
        if info_tag in vcf_variant.info:
            span = map(int, vcf_variant.info[info_tag].split(','))
            if len(span) != 2:
                raise ValueError('Invalid value for tag {0}. Require 2 values to adjust coordinates.'.format(info_tag))
            return (start + span[0], end + span[1])
        else:
            return (start, end)

    def convert(self, primary_variant, secondary_variant=None):
        '''
        Convert the passed VCF variant(s) into a BEDPE object
        '''
        vcf_variant = primary_variant
        if primary_variant is None:
            vcf_variant = secondary_variant

        try:
            sv_type = vcf_variant.info['SVTYPE']
        except KeyError:
            raise ValueError('SVTYPE field required for conversion to BEDPE')

        parser = self.simple_breakpoints
        if sv_type == 'BND':
            parser = self.bnd_breakpoints

        c1, s1, e1, c2, s2, e2, o1, o2 = parser(vcf_variant)

        s1, e1 = self.adjust_coordinate(vcf_variant, 'CIPOS', s1, e1)
        s2, e2 = self.adjust_coordinate(vcf_variant, 'CIEND', s2, e2)

        orig_name_a = vcf_variant.var_id
        orig_ref_a = vcf_variant.ref
        orig_alt_a = vcf_variant.alt
        info_a = vcf_variant.get_info_string()
        if primary_variant is None:
            info_a = "MISSING"
            orig_name_a = orig_ref_a = orig_alt_a = '.'
            c1, s1, e1, o1, c2, s2, e2, o2 = c2, s2, e2, o2, c1, s1, e1, o1

        info_b = '.'
        orig_name_b = orig_ref_b = orig_alt_b = '.'
        if sv_type == 'BND':
            if secondary_variant is None:
                info_b = "MISSING"
            else:
                info_b = secondary_variant.get_info_string()
                orig_name_b = secondary_variant.var_id
                orig_ref_b = secondary_variant.ref
                orig_alt_b = secondary_variant.alt

        # For MANTA single-ended BNDs, EVENT is not present.
        # XXX This has probably already been calculated outside of this method. May be a candidate to memoize or otherwise cache?
        # By adding to the variant class, perhaps?
        name = vcf_variant.var_id
        if 'EVENT' in vcf_variant.info:
            name = vcf_variant.info['EVENT']

        return Bedpe(map(str,[
            c1,
            max(s1, 0),
            max(e1, 0),
            c2,
            max(s2, 0),
            max(e2, 0),
            name,
            vcf_variant.qual,
            o1,
            o2,
            sv_type,
            vcf_variant.filter,
            orig_name_a,
            orig_ref_a,
            orig_alt_a,
            orig_name_b,
            orig_ref_b,
            orig_alt_b,
            info_a,
            info_b,
            vcf_variant.get_format_string(),
            vcf_variant.get_gt_string()]))




import unittest
import os
import sys
import gzip
from intervaltree_bio import GenomeIntervalTree, UCSCTable
import pandas as pd
sys.path.insert(0, '../')
import starseqr_utils as su


path = os.path.dirname(__file__)
if path != '':
    os.chdir(path)


class AnnoTestCase(unittest.TestCase):
    """Tests annotation"""
    mygtf = os.path.realpath("test_data/test.gtf.gz")
    genepred_annot = os.path.splitext(mygtf)[0] + ".genePred"
    ucsc_annot = os.path.splitext(mygtf)[0] + ".UCSCTable.gz"
    su.gtf_convert.gtf_to_genepred(mygtf, genepred_annot)
    su.gtf_convert.genepred_to_UCSCtable(genepred_annot, ucsc_annot)
    su.gtf_convert.gtf_to_genepred(mygtf, genepred_annot)
    su.gtf_convert.genepred_to_UCSCtable(genepred_annot, ucsc_annot)
    kg = gzip.open(ucsc_annot)
    global gtree
    gtree = GenomeIntervalTree.from_table(fileobj=kg, mode='tx', parser=UCSCTable.ENS_GENE)

    def test_get_jxnside_anno_v1(self):
        """test get_jxnside_anno"""
        jxn_filt = pd.DataFrame({'name': 'chr1:871160:+:chr1:985950:-:0:0',
                                 'jxn_reads': 'a1,a2,a3',
                                 'jxn_counts': 3,
                                 'spans': 5,
                                 'spanreads': 's1,s2,s3,s4,s5',
                                 'dist': 114790,
                                 'ann_format': 'Symbol:Transcript:Strand:Exon_No:Dist_to_Exon:Frame:CDS_Length'},
                                index=pd.Series(0))
        jxn_filt['left_symbol'], jxn_filt['left_annot'], jxn_filt['left_strand'], jxn_filt['left_cdslen'] = zip(*jxn_filt.apply(lambda x: su.annotate_sv.get_jxnside_anno(x['name'], gtree, 1), axis=1))
        jxn_filt['right_symbol'], jxn_filt['right_annot'], jxn_filt['right_strand'], jxn_filt['right_cdslen'] = zip(*jxn_filt.apply(lambda x: su.annotate_sv.get_jxnside_anno(x['name'], gtree, 2), axis=1))
        assert(jxn_filt['left_symbol'].iloc[0] == 'SAMD11')
        assert(jxn_filt['right_symbol'].iloc[0] == 'AGRN')
        assert(jxn_filt['left_strand'].iloc[0] == '+')
        assert(jxn_filt['right_strand'].iloc[0] == '+')
        assert(jxn_filt['left_cdslen'].iloc[0] == 9852)
        assert(jxn_filt['right_cdslen'].iloc[0] == 3395)

    def test_get_jxnside_anno_v2(self):
        """test get_jxnside_anno.. Tests where no transcripts found"""
        jxn_filt = pd.DataFrame({'name': 'chr19:560462:+:chr8:560462:-:0:0',
                                 'jxn_reads': 'a1,a2,a3',
                                 'jxn_counts': 3,
                                 'spans': 5,
                                 'spanreads': 's1,s2,s3,s4,s5',
                                 'dist': 13082084,
                                 'ann_format': 'Symbol:Transcript:Strand:Exon_No:Dist_to_Exon:Frame:CDS_Length'},
                                index=pd.Series(0))
        jxn_filt['left_symbol'], jxn_filt['left_annot'], jxn_filt['left_strand'], jxn_filt['left_cdslen']= zip(*jxn_filt.apply(lambda x: su.annotate_sv.get_jxnside_anno(x['name'], gtree, 1), axis=1))
        assert(jxn_filt['left_symbol'].iloc[0] == 'NA')
        assert(jxn_filt['left_strand'].iloc[0] == 'NA')
        assert(jxn_filt['left_cdslen'].iloc[0] == 'NA')

    def test_get_jxnside_anno_v3(self):
        """test get_jxnside_anno.. Tests non-coding transcripts for sorting"""
        jxn_filt = pd.DataFrame({'name': 'chr12:15704606:+:chr1:11918527:-:2:1',
                                 'jxn_reads': 'a1,a2,a3',
                                 'jxn_counts': 3,
                                 'spans': 5,
                                 'spanreads': 's1,s2,s3,s4,s5',
                                 'dist': 'NA',
                                 'ann_format': 'Symbol:Transcript:Strand:Exon_No:Dist_to_Exon:Frame:CDS_Length'},
                                index=pd.Series(0))
        jxn_filt['left_symbol'], jxn_filt['left_annot'], jxn_filt['left_strand'], jxn_filt['left_cdslen'] = zip(*jxn_filt.apply(lambda x: su.annotate_sv.get_jxnside_anno(x['name'], gtree, 1), axis=1))
        assert(jxn_filt['left_symbol'].iloc[0] == 'NA')
        assert(jxn_filt['left_strand'].iloc[0] == 'NA')
        assert(jxn_filt['left_cdslen'].iloc[0] == 'NA')

    def test_get_jxn_genes(self):
        """test_get_jxn_genes"""
        jxn_filt = pd.DataFrame({'name': 'chr1:871160:+:chr1:985950:-:0:0',
                                 'jxn_reads': 'a1,a2,a3',
                                 'jxn_counts': 3,
                                 'spans': 5,
                                 'spanreads': 's1,s2,s3,s4,s5',
                                 'dist': 114790,
                                 'ann_format': 'Symbol:Transcript:Strand:Exon_No:Dist_to_Exon:Frame:CDS_Length'},
                                index=pd.Series(0))
        jxn_filt['left_all'], jxn_filt['right_all'] = zip(*jxn_filt.apply(lambda x: su.annotate_sv.get_jxn_genes(x['name'], gtree), axis=1))
        assert(jxn_filt['left_all'].iloc[0] == ['SAMD11'])
        assert(jxn_filt['right_all'].loc[0] == ['AGRN'])

    def test_get_pos_genes(self):
        """test_get_pos_genes"""
        assert(su.annotate_sv.get_pos_genes('chr1', 871160, gtree) == ['SAMD11'])


if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python
# encoding: utf-8

from __future__ import (absolute_import, division, print_function)
import logging
import re
import starseqr_utils as su

logger = logging.getLogger('STAR-SEQR')


def find_unique_overhangs(reads_fq):
    '''get unique reads with flags 321, 337, 385, 401 in context of strand'''
    # +/+ = 401/321, +/- = 385/321, -/+ = 401/337, -/- = 385/337
    rfq_gen = su.common.FastqParser(reads_fq)
    res_left = set()
    res_right = set()
    for rfq in rfq_gen:
        read_tag = int(float(re.split('_', rfq.header)[-1]))
        if read_tag in [321, 337]:
            res_left.add(rfq.sequence)  # reads start from jxnright
        else:  # 385,401
            res_right.add(rfq.sequence)  # reads start from jxnleft
    return len(res_left) + len(res_right) # eventually break these down into separate


def get_diversity(jxn):
    clean_jxn = su.common.safe_jxn(jxn)
    jxn_dir = 'support' + '/' + clean_jxn + '/'
    overhang_fq = jxn_dir + 'overhang.fastq'
    overhang_res = find_unique_overhangs(overhang_fq)
    return overhang_res



import unittest
import os
import sys
import pandas as pd
sys.path.insert(0, '../')
import starseqr_utils as su

path = os.path.dirname(__file__)
if path != '':
    os.chdir(path)


class AnnoDBTestCase(unittest.TestCase):
    '''Tests annotation for db'''
    chimerdb3_path = su.common.find_resource('ChimerDB3.0_ChimerSeq.bedpe')
    fuca_path = su.common.find_resource('FusionCancer.bedpe')

    global chimerdb3
    chimerdb3 = pd.read_csv(chimerdb3_path, sep="\t", header=None, low_memory=False, engine='c')
    chimerdb3.columns = ['c1', 's1', 'e1', 'c2', 's2', 'e2', 'Fusion_pair', 'Counts',
                         'strand1', 'strand2', 'Source', 'Cancer', 'Features']

    global fuca
    fuca = pd.read_csv(fuca_path, sep="\t", header=None, low_memory=False, engine='c')
    fuca.columns = ['c1', 's1', 'e1', 'c2', 's2', 'e2', 'Database_ID', 'Rate',
                    'strand1', 'strand2', 'Methods', 'Cancers', 'Fusion_name']

    def test_annodb_v1(self):
        """CCDC6-RET fusion. This position has two overlapping breakpoints in chimerdb, 12 counts"""
        res1 = su.annotate_db.get_chimerdb('chr10:61665880:-:chr10:43612032:+:0:0', chimerdb3)
        print(res1)
        # (12, 'TCGA-PRADA,TCGA-TopHat-Fusion', 'LUAD,THCA', 'Oncogene,KB,Pub')
        assert(res1[0] == 12)
        assert("TCGA-PRADA" in res1[1].split(","))
        assert("THCA" in res1[2].split(","))

    def test_annodb_v1_2(self):
        """CCDC6-RET fusion. This position has two overlapping breakpoints in chimerdb, 12 counts"""
        res1 = su.annotate_db.get_fusioncancerdb('chr10:61665880:-:chr10:43612032:+:0:0', fuca)
        assert(res1 == (('', '', '')))

    def test_annodb_v2(self):
        """TMPRSS2-ERG Fusion. This position has two overlapping breakpoints in chimerdb, 300 counts, 3 methods"""
        res1 = su.annotate_db.get_chimerdb('chr21:42880007:-:chr21:39817543:+:0:0', chimerdb3)
        # (329, 'TCGA-FusionScan,TCGA-PRADA,TCGA-TopHat-Fusion', 'PRAD', 'KB,Pub')
        assert(res1[0] == 329)
        assert("TCGA-PRADA" in res1[1].split(","))
        assert("PRAD" in res1[2].split(","))

    def test_annodb_v3(self):
        """BCR-ABL Fusion. Found in both DBs"""
        res1 = su.annotate_db.get_chimerdb('chr22:23632600:-:chr9:133729450:+:0:0', chimerdb3)
        print(res1)
        # (3, 'TCGA-FusionScan,ChiTaRs', 'nan,LAML', 'Kinase,Oncogene,KB,Pub')
        assert(res1[0] == 3)
        assert("TCGA-FusionScan" in res1[1].split(","))
        assert("LAML" in res1[2].split(","))

    def test_annodb_v3_2(self):
        """BCR-ABL Fusion. Found in both DBs"""
        res1 = su.annotate_db.get_fusioncancerdb('chr22:23632600:-:chr9:133729450:+:0:0', fuca)
        # (0.85, 'tophat,chimerascan,SOAPfuse,FusionMap', 'Chronic_myelogenous_leukemia')
        assert(res1[0] == 0.85)
        assert("tophat" in res1[1].split(","))
        assert("Chronic_myelogenous_leukemia" in res1[2].split(","))


if __name__ == '__main__':
    unittest.main()



import unittest
import os
import sys
import gzip
sys.path.insert(0, '../')
import starseqr_utils as su

path = os.path.dirname(__file__)
if path != '':
    os.chdir(path)


class GTFTestCase(unittest.TestCase):
    '''Tests gtf conversion'''

    def test_gtf2genepred_v1(self):
        """test gtf to genepred"""
        mygtf = os.path.realpath("test_data/test.gtf.gz")
        genepred_annot = os.path.splitext(mygtf)[0] + ".genePred"
        os.remove(genepred_annot) if os.path.exists(genepred_annot) else None
        su.gtf_convert.gtf_to_genepred(mygtf, genepred_annot)
        assert os.path.exists(genepred_annot) == 1
        with open(genepred_annot) as myfile:
            count = sum(1 for line in myfile)
        assert(count) == 76

    def test_genepred2ucsc_v1(self):
        """test genepred to ucsc"""
        mygtf = os.path.realpath("test_data/test.gtf.gz")
        genepred_annot = os.path.splitext(mygtf)[0] + ".genePred"
        ucsc_annot = os.path.splitext(mygtf)[0] + ".UCSCTable.gz"
        os.remove(genepred_annot) if os.path.exists(genepred_annot) else None
        os.remove(ucsc_annot) if os.path.exists(ucsc_annot) else None
        su.gtf_convert.gtf_to_genepred(mygtf, genepred_annot)
        su.gtf_convert.genepred_to_UCSCtable(genepred_annot, ucsc_annot)
        assert os.path.exists(ucsc_annot) == 1
        with gzip.open(ucsc_annot, 'rb') as myfile:
            count = sum(1 for line in myfile)
        assert(count) == 76

    def test_genepred2ucsc_v2(self):
        """test genepred to ucsc. test for re-use"""
        mygtf = os.path.realpath("test_data/test.gtf.gz")
        genepred_annot = os.path.splitext(mygtf)[0] + ".genePred"
        ucsc_annot = os.path.splitext(mygtf)[0] + ".UCSCTable.gz"
        su.gtf_convert.gtf_to_genepred(mygtf, genepred_annot)
        su.gtf_convert.genepred_to_UCSCtable(genepred_annot, ucsc_annot)
        assert os.path.exists(ucsc_annot) == 1
        with gzip.open(ucsc_annot, 'rb') as myfile:
            count = sum(1 for line in myfile)
        assert(count) == 76


if __name__ == '__main__':
    unittest.main()



import re
import sys

class Bedpe(object):
    def __init__(self, bed_list):
        self.c1 = bed_list[0]
        self.s1 = int(bed_list[1])
        self.e1 = int(bed_list[2])
        self.c2 = bed_list[3]
        self.s2 = int(bed_list[4])
        self.e2 = int(bed_list[5])
        self.name = bed_list[6]
        self.score = self.parse_score(bed_list[7])
        self.o1 = bed_list[8]
        self.o2 = bed_list[9]
        self.svtype = bed_list[10]
        self.filter = bed_list[11]
        self.orig_name1 = bed_list[12]
        self.orig_ref1 = bed_list[13]
        self.orig_alt1 = bed_list[14]
        self.orig_name2 = bed_list[15]
        self.orig_ref2 = bed_list[16]
        self.orig_alt2 = bed_list[17]
        self.malformedFlag = 0
        self.info1 = bed_list[18]
        self.info2 = bed_list[19]
        self.misc = bed_list[20:]
        self.check_malformed()

        # FIXME This is only really needed for varlookup. Something more general would be helpful
        self.cohort_vars = dict()
        
        try:
            self.svtype = self.retrieve_svtype()
        except ValueError:
            sys.stderr.write('No SVTYPE parseable for {0}'.format('\t'.join(bed_list)))
            sys.exit(1)
        self.af = self.retrieve_af()
        if self.svtype != bed_list[10]:
            sys.stderr.write("SVTYPE at Column 11({0})) and SVTYPE in INFO Column({1}) don't match at variant ID {3}\n".format(str(bed_list[10]), str(self.svtype), self.name))

    @staticmethod
    def parse_score(score):
        if score.isdigit():
            return float(score)
        else:
            return score
    @property
    def info(self):
        '''
        Return the appropriate info field if only one is required. Info from the primary variant is preferred if available.
        '''
        if self.info1 == 'MISSING':
            return self.info2
        else:
            return self.info1

    def set_info(self, field, value):
        '''
        Add the info field to the BEDPE line info fields. As BEDPE lines don't know about their headers this is not a safe operation.
        Doesn't add to info field if it is the null character. Probably this is wrong.
        '''
        new_tag = ';' + str(field);
        if value is not None:
            new_tag += '=' + str(value)
        if self.malformedFlag != 1:
            self.info1 = self.info1 + new_tag
        if self.malformedFlag != 2 and self.info2 != '.':
            self.info2 = self.info2 + new_tag

    def check_malformed(self):
        if self.info1 == 'MISSING':
            self.malformedFlag = 1
        if self.info2 == 'MISSING':
            self.malformedFlag = 2      

    def retrieve_svtype(self):
        try:
            svtype = re.split('=', ''.join(filter(lambda x: 'SVTYPE=' in x, self.info.split(';'))))[1]
        except IndexError:
            raise ValueError
        return svtype

    def retrieve_af(self):
        try:
            af = re.split('=', ''.join(filter(lambda x: 'AF=' in x, self.info.split(';'))))[1]
        except IndexError:
            af = None
        return af
    
    def __str__(self):
        '''
        A string representation of the line represented by this object
        '''
        return '\t'.join([
            self.c1,
            str(self.s1),
            str(self.e1),
            self.c2,
            str(self.s2),
            str(self.e2),
            self.name,
            str(self.score),
            self.o1,
            self.o2,
            self.svtype,
            self.filter,
            self.orig_name1,
            self.orig_ref1,
            self.orig_alt1,
            self.orig_name2,
            self.orig_ref2,
            self.orig_alt2,
            self.info1,
            self.info2] + 
            self.misc
            )




import unittest
import os
import sys
sys.path.insert(0, '../')
import starseqr_utils as su

path = os.path.dirname(__file__)
if path != '':
    os.chdir(path)


class CrossHomologyTestCase(unittest.TestCase):
    '''Tests cross homology'''

    def test_crosshom_v1(self):
        """test homology paired.fastq TUBA1B--TUBA1A test case"""
        res1 = su.cross_homology.get_cross_homology('chr12:49521770:-:chr12:49578857:-:6:0')
        # print(res1)
        assert(res1 == (.75, 1.0))

    def test_crosshom_v2(self):
        """test homology paired.fastq no reads in fastq test case"""
        res1 = su.cross_homology.get_cross_homology('chr2:131389025:+:chr2:131996848:+:0:4')
        # print(res1)
        assert(res1 == (0, 0))

    def test_crosshom_v3(self):
        """test homology TRIM46--TMEM161A--a known FP with borderline homology """
        res1 = su.cross_homology.get_cross_homology('1:155152411:+:19:19243318:-:1:0')
        # print(res1)
        assert(res1 == (0.0, 0.5))

    def test_crosshom_v4(self):
        """test homology a known TP with borderline homology """
        res1 = su.cross_homology.get_cross_homology('5:149206449:+:1:156848913:pos:1:0')
        # print(res1)
        assert(res1 == (0.0, 0.0))


if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python

from __future__ import (absolute_import, division, print_function)
import sys
import time
from argparse import ArgumentParser
import pandas as pd


def parse_args():
    usage = " "
    parser = ArgumentParser(
        description="convert FusionCancer DB to bedpe:", epilog=usage)
    parser.add_argument('-i', '--file', type=str, required=True,
                        help='excel file from chimerdb3')
    parser.add_argument('-i2', '--file2', type=str, required=True,
                        help='excel file from chimerdb3 with fixed coordinates for chimerdb2 db')
    # parser.add_argument('-k', '--file3', type=str, required=True,
    #                     help='excel file from chimerdb3 for KB database')
    parser.add_argument('-o', '--out', type=str, required=False,
                        help='name of out file')
    return parser.parse_args()


def main():
    start = time.time()
    args = parse_args()
    chimerdb_seq = pd.read_excel(args.file, header=0)
    chimerdb_seq = chimerdb_seq[chimerdb_seq['Genome_Build_Version'] == 'hg19']
    # update names
    chimerdb_seq['Source'] = chimerdb_seq['Source'].str.replace('PRADA', 'TCGA-PRADA')
    chimerdb_seq['Source'] = chimerdb_seq['Source'].str.replace('FusionScan', 'TCGA-FusionScan')
    chimerdb_seq['Source'] = chimerdb_seq['Source'].str.replace('TopHat-Fusion', 'TCGA-TopHat-Fusion')
    # file2
    chimerdb_seq2 = pd.read_excel(args.file2, header=0)
    chimerdb_seq2 = chimerdb_seq2[chimerdb_seq2['Genome_Build_Version'] == 'hg19']
    chimerdb_all = pd.concat([chimerdb_seq, chimerdb_seq2])
    chimerdb_all['Cancer'] = chimerdb_all['Cancertype_or_disease'].str.replace(' ','_')
    # aggregate by left chr:pos and right chr:pos
    chimerdb_all.update(chimerdb_all[['ChimerKB', 'ChimerPub','Kinase' , 'Oncogene','Tumor_suppressor',
                                      'Receptor', 'Transcription_Factor', 'Cancer']].fillna('NA'))
    grouped_chimerdb = chimerdb_all.groupby(['H_chr', 'H_position', 'T_chr', 'T_position', 'Fusion_pair',
                                          'ChimerKB', 'ChimerPub','Kinase' , 'Oncogene','Tumor_suppressor',
                                          'Receptor', 'Transcription_Factor',
                                         ], as_index=True)
    df = grouped_chimerdb.agg({'Source': {'Source': lambda col: ','.join(set(col))},
                               'Cancer': {'Cancer': lambda col: ','.join(set(col))},
                               'BarcodeID': 'count'}).reset_index()
    df.columns = ['H_chr', 'H_position', 'T_chr', 'T_position', 'Fusion_pair', 'ChimerKB', 'ChimerPub',
                  'Kinase', 'Oncogene', 'Tumor_suppressor', 'Receptor', 'Transcription_Factor',
                  'Source', 'Counts', 'Cancer']
    # merge features into a single string
    df['Features'] = df[['Kinase', 'Oncogene', 'Tumor_suppressor', 'Receptor',
                         'Transcription_Factor', 'ChimerKB',
                         'ChimerPub']].apply(lambda x: ','.join([str(y) for y in x if y != 'NA']), axis=1)
    df.replace('', 'NA', inplace=True)

    # get columns for bedpe format
    df['c1'] = df['H_chr']
    df['c2'] = df['T_chr']
    df['s1'] = df['H_position']
    df['s2'] = df['T_position']
    df['s1'] = df['s1'].astype('int')
    df['s2'] = df['s2'].astype('int')
    df['e1'] = df['s1'] + 1
    df['e2'] = df['s2'] + 1
    df['strand1'] = '.'
    df['strand2'] = '.'
    df.sort_values(['c1', 's1'], ascending=[True, True], inplace=True)
    outcols = ['c1', 's1', 'e1', 'c2', 's2', 'e2', 'Fusion_pair', 'Counts', 'strand1', 'strand2', 'Source', 'Cancer', 'Features']

    df.to_csv(path_or_buf=args.out, header=False, sep='\t', columns=outcols, mode='w', index=False)

    end = time.time()
    elapsed = end - start
    print("Program took  %g seconds" % (elapsed))


if __name__ == "__main__":
    sys.exit(main())


#!/usr/bin/env python

from __future__ import print_function
import sys
import time
from argparse import ArgumentParser
import pandas as pd


def parse_args():
    usage = " "
    parser = ArgumentParser(
        description="convert FusionCancer DB to bedpe:", epilog=usage)
    parser.add_argument('-i', '--file', type=str, required=True,
                        help='excel file from FusionCancer')
    parser.add_argument('-o', '--out', type=str, required=False,
                        help='name of out file')
    return parser.parse_args()


def main():
    start = time.time()
    args = parse_args()
    df = pd.read_excel(args.file, header=0)

    df['c1'],  df['s1'] = zip(*df['Head_breakpoint_location'].str.split(':').tolist())
    df['c2'],  df['s2'] = zip(*df['Tail_breakpoint_location'].str.split(':').tolist())

    df['Fusion_name'] = df['Head_gene_symbol'].astype(str) + "--" + df['Tail_gene_symbol'].astype(str)
    df['Cancer_fix'] = df['Cancer_type'].str.replace(' ','_')
    grouped_fuca = df.groupby([ 'c1', 's1', 'c2', 's2', 'Database_ID',], as_index=True)
    fuca = grouped_fuca.agg({'Cancer_fix': {'Cancer_fix': lambda col: ','.join(set(col))},
                               'Detected_tools': {'Detected_tools': lambda col: ','.join(set(col))},
                               'Fusion_name': {'Fusion_name': lambda col: ','.join(set(col))},
                               'Rate': 'max'}).reset_index()
    fuca.columns = ['c1', 's1', 'c2', 's2', 'Database_ID', 'Detected_tools', 'Rate', 'Cancer_fix', 'Fusion_name']
    fuca['Methods'] = fuca['Detected_tools'].str.split(",", expand=False).apply(lambda x: ','.join(set(x)))
    fuca['Cancers'] = fuca['Cancer_fix'].str.split(",", expand=False).apply(lambda x: ','.join(set(x)))
    fuca.sort_values(['c1', 's1'], ascending=[True, True], inplace=True)

    fuca['s1'] = fuca['s1'].astype('int')
    fuca['s2'] = fuca['s2'].astype('int')
    fuca['e1'] = fuca['s1'] + 1
    fuca['e2'] = fuca['s2'] + 1
    fuca['strand1'] = '.'
    fuca['strand2'] = '.'

    outcols = ['c1', 's1', 'e1', 'c2', 's2', 'e2', 'Database_ID', 'Rate', 'strand1', 'strand2', 'Methods', 'Cancers', 'Fusion_name']
    fuca.to_csv(path_or_buf=args.out, header=False, sep='\t', columns=outcols, mode='w', index=False)

    end = time.time()
    elapsed = end - start
    print("Program took  %g seconds" % (elapsed))


if __name__ == "__main__":
    sys.exit(main())


import argparse
import sys
from subprocess import Popen, PIPE, STDOUT
from svtools.vcf.file import Vcf
import svtools.utils as su

def run_cnvnator(cnvnator_path, root, window, coord_list):
    # Read and run cnvnator
    p1 = Popen(['cat', coord_list.name], stdout=PIPE)
    cmd = map(str, [cnvnator_path, '-root', root, '-genotype', window])
    p2 = Popen(cmd, stdin=p1.stdout, stdout=PIPE)
    # NOTE the below awk command selects only those lines which were genotyped
    # The line beginning 'Assuming' looks to be the program reporting its assuming a male
    # Each line looks like
    # Genotype chr1:1-13000 5173T.root 0.377524 0.368896
    # The first number is the copy number for the requested window
    # The second seems to be the copy number for a fixed window size of 1000
    # See http://wiki.biouml.org/index.php/CNVnator_genotype_output_(file_format)
    p3 = Popen(['awk', '{ if($1!="Assuming"){print $4} }'], stdin=p2.stdout, stdout=PIPE)
    cn_list = map(float, p3.communicate()[0].split('\n')[:-1])
    return cn_list

def sv_readdepth(vcf_file, sample, root, window, vcf_out, cnvnator_path, coord_list):
    cn = run_cnvnator(cnvnator_path, root, window, coord_list)
    write_copynumber(vcf_file, sample, vcf_out, cn)

def write_copynumber(vcf_file, sample, vcf_out, cn_list):
    #go through the VCF and add the read depth annotations
    in_header = True
    header = []
    vcf = Vcf()
    i = 0
    s_index = -1
    for line in vcf_file:
        if in_header:
            if line[0] == '#' and line[1] == '#':
                header.append(line)
                continue
            if line[0] == '#' and line[1] != '#':
                  try:
                        s_index = line.rstrip().split('\t').index(sample)
                  except ValueError:
                        sys.stderr.write("Please input valid VCF, format field for " + sample + " not found in VCF")
                        sys.exit(1)
                  line = '\t'.join(map(str, line.rstrip().split('\t')[:9] + [sample]))
                  header.append(line)
                  continue
            else:
                in_header = False
                vcf.add_header(header)
                vcf.add_format('CN', 1, 'Float', 'Copy number of structural variant segment.')
                vcf_out.write(vcf.get_header() + '\n')
        v = line.rstrip().split('\t')
        # XXX Is this second check necessary? Wouldn't this be handled above? Missing header would hit this?
        if s_index == -1:
            sys.stderr.write("Input a valid sample name: " + sample + " not found in a provided VCF")
            sys.exit(1)
        v = v[:9] + [v[s_index]]
        if not any("SVTYPE=BND" in s for s in v):
            if "CN" not in v[8]:
                v[8] = v[8] + ":CN"
                v[9] = v[9] + ":" + str(cn_list[i])
            else:
                cn_index = v[8].rstrip().split(":").index("CN")
                gts = v[9].rstrip().split(":")
                gts[cn_index] = str(cn_list[i])
                v[9] = ":".join(gts)
            i += 1
        # write the VCF
        vcf_out.write('\t'.join(v) + '\n')
    vcf_out.close()
    return

def description():
    return 'add copynumber information using cnvnator-multi'

def epilog():
    return '''As this program runs cnvnator-multi you must provide its location and must remember to have the ROOT package installed and properly configured. The input VCF file may be gzipped. If the input VCF file is omitted then the tool reads from stdin. Note that the coordinates file must end with a line containing the word exit.'''

def add_arguments_to_parser(parser):
    parser.add_argument('-c', '--coordinates', metavar='<FILE>', type=argparse.FileType('r'), required=True, default=None, help='file containing coordinate for which to retrieve copynumber (required)')
    parser.add_argument('-r', '--root', metavar='<FILE>', required=True, help='CNVnator .root histogram file (required)')
    parser.add_argument('-w', '--window', metavar='<INT>', required=True, help='CNVnator window size (required)')
    parser.add_argument('-s', '--sample', metavar='<STRING>', required=True, help='sample to annotate (required)')
    parser.add_argument('--cnvnator', metavar='<PATH>', required=True, help='path to cnvnator-multi binary (required)')
    parser.add_argument('-v', '--input-vcf', metavar='<VCF>', default=None, help='VCF input')
    parser.add_argument('-o', '--output-vcf', metavar='<PATH>', type=argparse.FileType('w'), default=sys.stdout, help='output VCF to write (default: stdout)')
    parser.set_defaults(entry_point=run_from_args)

def command_parser():
    parser = argparse.ArgumentParser(description=description(), epilog=epilog())
    add_arguments_to_parser(parser)
    return parser

def run_from_args(args):
    with su.InputStream(args.input_vcf) as stream:
        sv_readdepth(stream, args.sample, args.root, args.window, args.output_vcf, args.cnvnator, args.coordinates)

# initialize the script
if __name__ == '__main__':
    parser = command_parser()
    args = parser.parse_args()
    sys.exit(args.entry_point(args))



import unittest
import os
import sys
sys.path.insert(0, '../')
import starseqr_utils as su

path = os.path.dirname(__file__)
if path != '':
    os.chdir(path)


class ReadSupportTestCase(unittest.TestCase):
    '''Tests read support'''

    def test_readsupport_v1(self):
        '''v1'''
        pass


if __name__ == '__main__':
    unittest.main()



import os
import sys
sys.path.insert(0, '../')
import starseqr_utils as su
import unittest
import shutil
import pandas as pd


path = os.path.dirname(__file__)
if path != '':
    os.chdir(path)


class ExonSeqTestCase(unittest.TestCase):
    '''Tests exon seq retrieval'''

    def setUp(self):
        jxn = 'chr1:10:+:chr2:20:+:0:0'
        clean_jxn = su.common.safe_jxn(jxn)
        jxn_dir = 'support' + '/' + clean_jxn + '/'
        shutil.rmtree(jxn_dir, ignore_errors=True)
        os.mkdir(jxn_dir)

    def tearDown(self):
        jxn = 'chr1:10:+:chr2:20:+:0:0'
        clean_jxn = su.common.safe_jxn(jxn)
        jxn_dir = 'support' + '/' + clean_jxn + '/'
        shutil.rmtree(jxn_dir, ignore_errors=True)

    def test_exons2seq_v1(self):
        """test_exons2seq + strand"""
        fa_path = 'test_data/ex1.fa'
        exons = [[[[1, 'chr1', 1, 10, '+', 'ENST1'], [2, 'chr1', 11, 20, '+', 'ENST1'], [3, 'chr1', 21, 30, '+', 'ENST1']],
                  [[1, 'chr1', 1, 10, '-', 'ENST2'], [2, 'chr1', 21, 30, '-', 'ENST2'], [3, 'chr1', 31, 40, '-', 'ENST2']]]]
        df = pd.DataFrame({'name': 'chr1:10:+:chr2:20:+:0:0', 'left_trx_exons': exons})
        df['seqs'] = df.apply(lambda x: su.core.exons2seq(fa_path, x['left_trx_exons'], x['name'], "left"), axis=1)
        res2 = df['seqs'][0][0][1]
        assert(res2[1] == 'TGGACGAGTAAACCACACGCCACTAGT')

    def test_exons2seq2_v2(self):
        """test_exons2seq + strand"""
        fa_path = 'test_data/ex1.fa'
        exons = [[[[1, 'chr1', 1, 10, '+', 'ENST1'], [2, 'chr1', 11, 20, '+', 'ENST1'], [3, 'chr1', 21, 30, '+', 'ENST1']],
                  [[1, 'chr1', 1, 10, '-', 'ENST2'], [2, 'chr1', 21, 30, '-', 'ENST2'], [3, 'chr1', 31, 40, '-', 'ENST2']]]]
        df = pd.DataFrame({'name': 'chr1:10:+:chr2:20:+:0:0', 'left_trx_exons': exons})
        df['seqs'] = df.apply(lambda x: su.core.exons2seq(fa_path, x['left_trx_exons'], x['name'], "left"), axis=1)
        res2 = df['seqs'][0][0][1]
        assert(res2[1] == 'TGGACGAGTAAACCACACGCCACTAGT')

    def test_exons2seq2_v3(self):
        """test_exons2seq for fusions"""
        fa_path = 'test_data/ex1.fa'
        exons = [[[[1, 'chr1', 1, 10, '+', 'ENST1'], [2, 'chr1', 11, 20, '+', 'ENST1'], [3, 'chr1', 21, 30, '+', 'ENST1']],
                  [[1, 'chr1', 1, 10, '-', 'ENST2'], [2, 'chr1', 21, 30, '-', 'ENST2'], [3, 'chr1', 31, 40, '-', 'ENST2']]]]
        exons2 = [[[[1, 'chr2', 1, 10, '+', 'ENST3'], [2, 'chr2', 11, 20, '+', 'ENST3'], [3, 'chr2', 21, 30, '+', 'ENST3']],
                   [[1, 'chr2', 1, 10, '-', 'ENST4'], [2, 'chr2', 21, 30, '-', 'ENST4'], [3, 'chr2', 31, 40, '-', 'ENST4']]]]
        df = pd.DataFrame({'name': 'chr1:10:+:chr2:20:+:0:0', 'left_exons': exons, 'right_exons': exons2})
        df['all_fusion_seqs'] = df.apply(lambda x: su.core.exons2seq(fa_path, x['left_exons'], x['name'], "all_fusion", x['right_exons']), axis=1)
        res2 = df['all_fusion_seqs'][0][0][1]
        assert(res2[0].split('|')[0] == 'ENST1--ENST4')
        assert(res2[1] == 'ACTAGTGGCCATTGTAAAGTGTGGTTTTTTCTTAAAGAATTTTTCTTCATTTGA')


    def test_exons2seq_v4(self):
        """test_exons2seq: tests empty"""
        fa_path = 'test_data/ex1.fa'
        exons = ['NA']
        df = pd.DataFrame({'name': 'chr1:10:+:chr2:20:+:0:0', 'left_trx_exons': exons})
        df['seqs'] = df.apply(lambda x: su.core.exons2seq(fa_path, x['left_trx_exons'], x['name'], "left"), axis=1)
        assert(df['seqs'].iloc[0] == None)


if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python
# encoding: utf-8

from __future__ import (absolute_import, division, print_function)
import six
from six import reraise as raise_
import os
import sys
import re
import time
import logging
import subprocess as sp
import pysam  # requires 0.9.0
import collections
import numpy as np
import starseqr_utils as su

try:
    maketrans = ''.maketrans
except AttributeError:
    # fallback for Python 2
    from string import maketrans

logger = logging.getLogger('STAR-SEQR')


def find_support_reads(jxn, bam, side, tx, sub_reads, gtree):
    chrom1, pos1, str1, chrom2, pos2, str2, repleft, repright = re.split(':', jxn)
    pos1 = int(pos1)
    pos2 = int(pos2)

    if side == 2:
        chrom1, chrom2 = chrom2, chrom1
        pos1, pos2 = pos2, pos1
        flipstr = maketrans('-+', '+-')
        str1, str2 = str2.translate(flipstr), str1.translate(flipstr)
        repleft, repright = repright, repleft

    maxhom = max(int(repright), int(repleft))
    if str1 == '+':
        pos1left = pos1 - 100000
        pos1right = pos1 + maxhom + 10
    if str2 == '-':
        pos2left = pos2 - 100000
        pos2right = pos2 + maxhom + 10
    if str1 == '-':
        pos1left = pos1 - maxhom - 10
        pos1right = pos1 + 100000
    if str2 == '+':
        pos2left = pos2 - maxhom - 10
        pos2right = pos2 + 100000

    # define all combinations upfront for recordkeeping
    retDict = {}
    lmode = ['spanleft', 'spanright', 'jxnleft', 'jxnright', 'hangleft', 'hangright']
    lstrand = ['for', 'rev']
    lorient = ['first', 'second']
    for x in lmode:
        retDict[x] = {}
        for y in lstrand:
            for z in lorient:
                retDict[x][(y, z)] = {}

    for read in bam:
        mymode = ''
        # determine if span, jxn read or chimeric overhang
        if (read.next_reference_name == chrom2 and read.reference_name == chrom1 and
                not (read.flag & 2) and not (read.flag & 256)):
            if ((int(read.reference_start) > pos1left and
                 int(read.reference_start) < pos1right) and
                (int(read.next_reference_start) > pos2left and
                 int(read.next_reference_start) < pos2right)):
                mymode = 'span'
        elif (read.next_reference_name == read.reference_name and
                (read.flag & 2) and not (read.flag & 256)):
            if ((int(read.reference_start) > pos1left and
                 int(read.reference_start) < pos1right) or
                (int(read.next_reference_start) > pos2left and
                 int(read.next_reference_start) < pos2right)):
                mymode = 'jxn'
        elif (read.next_reference_name == chrom2 and read.reference_name == chrom1 and
                not (read.flag & 2) and (read.flag & 256)):
            if ((int(read.reference_start) > pos1left and
                 int(read.reference_start) < pos1right) or
                (int(read.next_reference_start) > pos2left and
                 int(read.next_reference_start) < pos2right)):
                mymode = 'hang'

        if mymode:
            myside = 'left' if side == 1 else 'right'
            mytype = mymode + myside
            mystrand = 'for' if (read.flag & 16) else 'rev'
            myorient = 'first' if (read.flag & 64) else 'second'
            is_subset = 'SUB' if read.query_name in sub_reads else 'ALL'
            if is_subset == 'SUB':
                read_tx = su.annotate_sv.get_pos_genes(read.reference_name, read.reference_start, gtree)
                read_tx2 = su.annotate_sv.get_pos_genes(read.next_reference_name, read.next_reference_start, gtree)
                if set(tx).intersection(set(read_tx)) and set(tx).intersection(set(read_tx2)):
                    retDict[mytype][(mystrand, myorient)][read.query_name] = (is_subset,
                                                                              read.get_tag('AS'),
                                                                              read.get_tag('nM'),
                                                                              read.query_alignment_length,
                                                                              int(np.mean(read.query_alignment_qualities)))
    return retDict


def get_reads_from_bam(bam_file, jxn, tx, s_reads, gtree):
    '''
    Fetch reads from each direction of jxn and get an accounting of each if unique or duplicate.
    '''
    logger.debug('Extracting supporting read information for ' + jxn)
    chrom1, pos1, str1, chrom2, pos2, str2, repleft, repright = re.split(':', jxn)

    if not os.path.exists(bam_file):
        logger.error('BAM file could not be found: ' + bam_file)
        sys.exit()
    bamObject = pysam.Samfile(bam_file, 'rb')
    if not bamObject.check_index():
        logger.error('Index file could not be found for: ' + bam_file)
        sys.exit()

    # account for genome boundaries
    len1 = six.next((item for item in bamObject.header['SQ'] if item['SN'] == chrom1))['LN']
    len2 = six.next((item for item in bamObject.header['SQ'] if item['SN'] == chrom2))['LN']
    dist1_less = max(i for i in [int(pos1) - 100000, 0] if i >= 0)
    dist1_plus = min(i for i in [int(pos1) + 100000, len1] if i >= 0)
    dist2_less = max(i for i in [int(pos2) - 100000, 0] if i >= 0)
    dist2_plus = min(i for i in [int(pos2) + 100000, len2] if i >= 0)

    results = {}
    # support left
    logger.debug('Extracting left paired reads for ' + jxn)
    bam = bamObject.fetch(chrom1, dist1_less, dist1_plus)
    support_for = find_support_reads(jxn, bam, 1, tx, s_reads, gtree)
    results['spanleft'] = support_for['spanleft']
    results['jxnleft'] = support_for['jxnleft']
    results['hangleft'] = support_for['hangleft']
    # support right
    logger.debug('Extracting right paired reads for ' + jxn)
    bam = bamObject.fetch(chrom2, dist2_less, dist2_plus)
    support_rev = find_support_reads(jxn, bam, 2, tx, s_reads, gtree)
    results['spanright'] = support_rev['spanright']
    results['jxnright'] = support_rev['jxnright']
    results['hangright'] = support_rev['hangright']
    bamObject.close()
    logger.debug('Finished extracting supporting read information for ' + jxn)
    return results


def subset_bam_by_reads(bam, out_bam, read_ids, jxn):
    # bamfilternames is sensitive to duplicate read names in the list. Make sure they are unique.
    logger.debug('Subset bam with supporting reads to ' + out_bam)
    names = 'names=' + read_ids
    index = 'index=1'
    indexfilename = 'indexfilename=' + out_bam + '.bai'
    tmpfile = 'tmpfile=' + out_bam + '.tmp'
    stdinfile = bam
    bamf = open(out_bam, 'wb')
    try:
        with open(stdinfile, 'rb') as f:
            retcode = sp.call(['bamfilternames', names, index, indexfilename,
                               tmpfile], stdin=f, stdout=bamf, stderr=sp.PIPE)
        bamf.close()
    except OSError as o:
        bamf.close()
        logger.error('bamfilternames Failed', exc_info=True)
        logger.error('Exception: ' + str(o))
        traceback = sys.exc_info()[2]
        raise_(ValueError, e, traceback)
    return


def bam2fastq(in_bam, bam_type, jxn_dir):
    logger.debug('Converting bams to fastqs')
    if not os.path.exists(in_bam):
        logger.error('BAM file could not be found: ' + in_bam)
        sys.exit(1)
    try:
        bam_nsort = in_bam[:-4] + '.nsorted.bam'
        pysam.sort('-n', in_bam, '-o', bam_nsort, catch_stdout=False)
    except (OSError) as o:
        logger.error('Exception: ' + str(o))
        logger.error('pysam Failed!', exc_info=True)
        traceback = sys.exc_info()[2]
        raise_(ValueError, e, traceback)
    bamObject = pysam.Samfile(bam_nsort, 'rb')

    if bam_type == "span":
        out_fq = open(jxn_dir + 'span.fastq', 'w')
        for read in bamObject.fetch(until_eof=True):
            if read.is_read1:
                orient = 1
                if read.is_reverse:
                    seq = su.common.rc(read.query_alignment_sequence)
                    quals = read.query_alignment_qualities[::-1]
                else:
                    seq = read.query_alignment_sequence
                    quals = read.query_alignment_qualities
            else:
                orient = 2
                if read.is_reverse:
                    seq = su.common.rc(read.query_alignment_sequence)
                    quals = read.query_alignment_qualities[::-1]
                else:
                    seq = read.query_alignment_sequence
                    quals = read.query_alignment_qualities
            out_fq.write('@' + read.query_name + '/' + str(orient) + '\n')
            out_fq.write(seq + '\n')
            out_fq.write('+' + '\n')
            out_fq.write(''.join(
                list(map(chr, [x + 33 for x in quals]))) + '\n')
        out_fq.close()
        os.remove(bam_nsort)
    elif bam_type == "split":
        out_fq = open(jxn_dir + 'split.fastq', 'w')
        out_fq2 = open(jxn_dir + 'overhang.fastq', 'w')
        for read in bamObject.fetch(until_eof=True):
            # use full junction sequence for split reads...accomodates assembly
            out_fq.write('@' + read.query_name + '_' + str(read.flag) + '\n')
            out_fq.write(read.query_sequence + '\n')
            out_fq.write('+' + '\n')
            out_fq.write(''.join(
                list(map(chr, [x + 33 for x in read.query_qualities]))) + '\n')
            # write overhang separately
            if read.flag & 256:
                out_fq2.write('@' + read.query_name + '_' + str(read.flag) + '\n')
                out_fq2.write(read.query_alignment_sequence + '\n')
                out_fq2.write('+' + '\n')
                out_fq2.write(''.join(
                    list(map(chr, [x + 33 for x in read.query_alignment_qualities]))) + '\n')
        out_fq.close()
        out_fq2.close()
        os.remove(bam_nsort)
    return


def get_rna_support(jxn, tx, s_reads, in_bam, gtree):
    ''' Run command to identify read support for a single breakpoint '''
    logger.debug('Getting read support for ' + jxn)
    start = time.time()
    clean_jxn = su.common.safe_jxn(jxn)
    jxn_dir = 'support' + '/' + clean_jxn + '/'
    su.common.make_new_dir(jxn_dir)
    # get supporting read counts and other metrics
    s_reads = s_reads.split(',')
    extracted = get_reads_from_bam(in_bam, jxn, tx, s_reads, gtree)
    logger.debug('Found and extracted reads successfully')
    results = collections.defaultdict(list)
    results['name'] = jxn
    span_reads = []
    split_reads = []
    for ktype in extracted:
        for ktuple in extracted[ktype]:
            kstrand, korient = ktuple
            dname = ktype + '_' + kstrand + '_' + korient
            found_reads = list(set(extracted[ktype][ktuple].keys()))
            results[dname + "_reads"] = found_reads
            results[dname] = len(results[dname + "_reads"])
            for kname, val in six.iteritems(extracted[ktype][ktuple]):
                if str(ktype).startswith("span"):
                    span_reads.append(kname)
                elif str(ktype).startswith("jxn"):
                    split_reads.append(kname)
                vun, vas, vmm, vseqlen, vmeanbq = val  # metric values: read flag, AS, nM, length, qualities
                results[dname + '_AS'].append(vas)
                results[dname + '_mismatches'].append(vmm)
                results[dname + '_seqlen'].append(vseqlen)
                results[dname + '_meanBQ'].append(vmeanbq)
            # convert lists to strings for easier record keeping
            results[dname + '_reads'] = ','.join(list(map(str, results[dname + '_reads'])))
            results[dname + '_AS'] = ','.join(list(map(str, results[dname + '_AS'])))
            results[dname + '_mismatches'] = ','.join(list(map(str, results[dname + '_mismatches'])))
            results[dname + '_seqlen'] = ','.join(list(map(str, results[dname + '_seqlen'])))
            results[dname + '_meanBQ'] = ','.join(list(map(str, results[dname + '_meanBQ'])))


    # write unique reads to file for subsetting bam later
    # TODO: Consider making one bam and dividing after.
    # span reads
    spanread_path = jxn_dir + 'span_readids.txt'
    span_fh = open(spanread_path, 'w')
    for uread in set(span_reads):
        span_fh.write(uread + '\n')
    span_fh.close()
    # split reads
    splitread_path = jxn_dir + 'split_readids.txt'
    split_fh = open(splitread_path, 'w')
    for uread in set(split_reads):
        split_fh.write(uread + '\n')
    split_fh.close()

    # subset supporting reads to bam
    logger.debug('Subsetting reads from bam')
    # span
    spanbam = jxn_dir + 'span.bam'
    subset_bam_by_reads(in_bam, spanbam, spanread_path, jxn)
    pysam.index(spanbam)
    # split
    splitbam = jxn_dir + 'split.bam'
    subset_bam_by_reads(in_bam, splitbam, splitread_path, jxn)
    pysam.index(splitbam)

    # convert bam support to fastqs
    bam2fastq(spanbam, "span", jxn_dir)
    bam2fastq(splitbam, "split", jxn_dir) # also produces overhang.fastq

    logger.info(jxn + ' support took  %g seconds' % (time.time() - start))
    return results



from abc import ABCMeta
import sys
import os
import subprocess
import signal
import pkg_resources as pkg

class ExternalCmd(object):
    __metaclass__ = ABCMeta

    def __init__(self, name, resource_path):
        self.name = name
        self.resource_path = resource_path

    def path_to_shell_script(self):
        path_to_script = pkg.resource_filename(__name__, self.resource_path)
        if os.path.isfile(path_to_script):
            return path_to_script
        else:
            sys.stderr.write('Unable to locate {0} script at {1}\n'.format(self.name, path_to_script))
            sys.exit(1)
    
    def run_cmd_with_options(self, options):
        cmd = [ self.path_to_shell_script() ]
        cmd.extend(options)
        # Here we re-set Python's treatment of SIGPIPE to the default
        # as described here: http://www.chiark.greenend.org.uk/~cjwatson/blog/python-sigpipe.html
        sys.stderr.write('Running {1} with options: {0}\n'.format(' '.join(cmd), self.name))
        p = subprocess.Popen(cmd, preexec_fn=lambda:
                signal.signal(signal.SIGPIPE, signal.SIG_DFL))
        code = p.wait()
        # The check for code != 141 is here because 
        # 141 indicates a SIGPIPE signal returned in the underlying bash pipelines
        # We want to be silent there
        # FIXME 141 is bash specific and while the underlying scripts are bash
        # It is not clear that the shell this script is run in should be exiting with 141 
        # or if that is even necessary
        if code:
            if code is not 141:
                sys.stderr.write('{0} exited with code {1}\n'.format(self.name, code))
            sys.exit(code)



import starseqr_utils.common
import starseqr_utils.core
import starseqr_utils.gtf_convert
import starseqr_utils.annotate_sv
import starseqr_utils.support_funcs_rna
import starseqr_utils.run_primer3
import starseqr_utils.star_funcs
import starseqr_utils.cross_homology
import starseqr_utils.overhang_diversity
import starseqr_utils.run_assembly
import starseqr_utils.annotate_db
import starseqr_utils.sv2bedpe

__version__ = "0.3.0"



#!/usr/bin/env python
# encoding: utf-8

from __future__ import (absolute_import, division, print_function)
from six.moves import zip
import six
from six import reraise as raise_
import os
import sys
import re
import gzip
import logging
import time
import errno
import pandas as pd
import numpy as np
from itertools import groupby, repeat
import multiprocessing as mp
import signal
import pysam
from intervaltree_bio import GenomeIntervalTree
import starseqr_utils as su

logger = logging.getLogger('STAR-SEQR')

try:
    maketrans = ''.maketrans
except AttributeError:
    # fallback for Python 2
    from string import maketrans


def check_file_exists(path):
    if (os.stat(os.path.realpath(path)).st_size == 0):
        logger.error("Exiting. Cannot find file: " + os.path.abspath(path))
        sys.exit(1)


def is_newer_than(target_path, orig_path):
    if not os.path.isfile(target_path):
        return False
    return (os.stat(target_path).st_mtime >= os.stat(orig_path).st_mtime)


def find_resource(filename):
    packagedir = su.__path__[0]
    dirname = os.path.join(packagedir, 'resources')
    fullpath = os.path.abspath(os.path.join(dirname, filename))
    return fullpath


def make_new_dir(newdir):
    try:
        os.mkdir(newdir)
    except OSError as e:
        if e.errno != errno.EEXIST:
            logger.error("Exception: " + str(e))
            raise e
        pass


def force_symlink(file1, file2):
    try:
        os.symlink(file1, file2)
    except OSError as e:
        if e.errno == errno.EEXIST:
            os.remove(file2)
            os.symlink(file1, file2)


def which(program):
    def is_exe(fpath):
        return os.path.isfile(fpath) and os.access(fpath, os.X_OK)
    fpath, fname = os.path.split(program)
    if fpath:
        if is_exe(program):
            return program
    else:
        for path in os.environ["PATH"].split(os.pathsep):
            path = path.strip('"')
            exe_file = os.path.join(path, program)
            if is_exe(exe_file):
                return exe_file
    return None


def pandas_parallel(df, func, nthreads, *opts):
    '''wrapper to run pandas apply func using map. *opts will take any number of optional arguments to be passed
    into the function. Note these will be passed to the function as a tuple and need to be parsed. '''
    start = time.time()

    def init_worker():
        signal.signal(signal.SIGINT, signal.SIG_IGN)
    try:
        df_split = np.array_split(df, min(nthreads, len(df.index)))
        pool = mp.Pool(nthreads, init_worker)
        if not opts:
            tuple_opts = df_split
        if len(opts) == 1:
            tuple_opts = zip(df_split, repeat(opts[0]))
        elif len(opts) > 1:
            tuple_opts = zip(df_split, repeat(opts))
        these_res = pool.map(func, tuple_opts)
        df = pd.concat(these_res)
        pool.close()
        pool.join()
    except KeyboardInterrupt as e:
        logger.error("Error: Keyboard interrupt")
        pool.terminate()
        raise e
    except Exception as e:
        logger.error("Exception: " + str(e))
        pool.terminate()
        traceback = sys.exc_info()[2]
        raise_(ValueError, e, traceback)
    logger.info("Time to run pandas_parallel on " + str(func.__name__) + " took %g seconds" % (time.time() - start))
    return df


def safe_jxn(jxn):
    ''' make a jxn string safe for folder and filenames'''
    fix_jxn = str(jxn).replace(':', '_')
    fix_jxn = str(fix_jxn).replace('+', 'pos')
    fix_jxn = str(fix_jxn).replace('-', 'neg')
    return fix_jxn


def bed_to_tree(bed):
    with open(bed, 'r') as f:
        btree = GenomeIntervalTree.from_bed(fileobj=f)
    return btree


def subset_bed_func(jxn, targets_tree, sub_style='either'):
    chrom1, pos1, str1, chrom2, pos2, str2, repleft, repright = re.split(':', jxn)
    intersect1 = len(targets_tree[str(chrom1)].search(int(pos1)))
    intersect2 = len(targets_tree[str(chrom2)].search(int(pos2)))
    if sub_style == 'either':
        if intersect1 or intersect2 >= 1:
            return 1
        else:
            return 0
    elif sub_style == 'both':
        if intersect1 >= 1 and intersect2 >= 1:
            return 1
        else:
            return 0


def rc(dna):
    ''' reverse complement '''
    complements = maketrans('acgtrymkbdhvACGTRYMKBDHV', 'tgcayrkmvhdbTGCAYRKMVHDB')
    return dna.translate(complements)[::-1]


def fasta_iter(fasta_name):
    ''' Given a fasta file, yield tuples of header, sequence'''
    fh = open(fasta_name)
    faiter = (x[1] for x in groupby(fh, lambda line: line[0] == '>'))
    for header in faiter:
        header = six.next(header)[1:].strip()  # drop the '>'
        # join all sequence lines to one.
        seq = ''.join(s.strip() for s in six.next(faiter))
        yield header, seq
    fh.close()


class FastqRead(object):
    """Represents 1 read, or 4 lines of a casava 1.8-style fastq file."""
    def __init__(self, file_obj):
        header = six.next(file_obj).rstrip()
        assert header.startswith('@')
        self.header = header
        self.sequence = six.next(file_obj).rstrip()
        self.seq_len = len(self.sequence)
        six.next(file_obj)
        self.quality = six.next(file_obj).rstrip()

    def __str__(self):
        return "{0}\n{1}\n+\n{2}\n+\n{3}\n".format(self.header, self.sequence,
                                                   self.quality, self.seq_len)


class FastqParser(object):
    """Parses a fastq file into FastqReads."""
    def __init__(self, filename):
        if filename.endswith('.gz'):
            self._file = gzip.open(filename, 'rb')
        else:
            self._file = open(filename, 'r')
        self._line_index = 0

    def __iter__(self):
        return self

    def __next__(self):
        read = FastqRead(self._file)
        if read:
            self._line_index += 4
            return read
        else:
            self._file.close()
            raise StopIteration()

    next = __next__  # Python 2


def sam_2_coord_bam(in_sam, out_bam, nthreads):
    ''' convert a sam to coordinate sorted bam '''
    if (out_bam[-4:] == ".bam"):
        bam_prefix = str(out_bam[:-4])
    else:
        bam_prefix = str(out_bam)
    bam_unsort = bam_prefix + ".unsorted.bam"
    pysam.view("-Sbu", "-@", str(nthreads), "-o", str(bam_unsort), str(in_sam), catch_stdout=False)
    pysam.sort(str(bam_unsort), "-@", str(nthreads), "-o", str(bam_prefix + ".bam"), catch_stdout=False)
    pysam.index("%s.bam" % bam_prefix, catch_stdout=False)
    os.remove(bam_unsort)


def bam_2_nsort_sam(in_bam, out_sam, nthreads):
    ''' convert a bam to a name sorted sam '''
    bam_sort = str(in_bam[:-4] + ".nsorted")
    pysam.sort("-n", "-@", str(nthreads), str(in_bam), "-o", str(bam_sort + ".bam"), catch_stdout=False)
    pysam.view("-h", "-@", str(nthreads), "-o", str(out_sam), str(bam_sort + ".bam"), catch_stdout=False)
    os.remove(bam_sort + ".bam")
